/* Encyklopedie jazyka C
 *
 * Modul s pomocnými funkcemi pro tisk číselných hodnot v bitové podobě.
 *
 * 2023-08-19-22.39
 * 2023-08-15-18.31
 */

/* Copyright (c) David Martinek, 2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

#include "bitovy-tisk.h"
#include <stdio.h>
#include <stdint.h>

void tiskni8Bitu(char bajt)
{
  unsigned char maska = (unsigned char)0x80; // 1000 0000
  while (maska != 0)
  {
    // (bajt & maska) vymaskuje bajt maskou
    // => pokud je v bajtu na místě bitu v masce logická 1,
    //    výsledkem bude nenulová hodnota
    putchar( (bajt & maska) ? '1' : '0');
    maska = maska >> 1; // bitovy posuv vpravo
  }
}

void tiskniNBitu(unsigned long cislo, unsigned char n, unsigned char zarovnani)
{
  unsigned long maska = 1l << (n-1); // 0000 0001 << n

  for (unsigned char i = 0; i < zarovnani-n; ++i)
  {
    putchar(' ');
  }

  while (maska != 0)
  {
    // (cislo & maska) vymaskuje bajt maskou
    // => pokud je v bajtu na místě bitu v masce logická 1,
    //    výsledkem bude nenulová hodnota
    putchar( (cislo & maska) ? '1' : '0');
    maska = maska >> 1; // bitovy posuv vpravo
  }
}

typedef union
{
  float cislo;
  uint32_t bity;
} Tfloathack;


void tiskniBityFloat(float cislo)
{
  Tfloathack x;
  x.cislo = cislo;
//  unsigned long bity = *(unsigned long *)&cislo; // to je ale hnusný hack, co?
  uint32_t maska = 1l << (sizeof(float)*8-1); // 0000 0001 << delka-1

  // znaménkový bit
  putchar((x.bity & maska) ? '1' : '0');
  putchar('|');

  // exponent - v typu float má 8 bitů
  maska >>= 1;
  for (int i = 0; i < 8; ++i)
  {
    putchar((x.bity & maska) ? '1' : '0');
    maska >>= 1;
  }

  // mantisa - v typu float má 23 bitů
  putchar('|');
  for (int i = 0; i < 23; ++i)
  {
    putchar((x.bity & maska) ? '1' : '0');
    maska >>= 1;
  }

  /* Hack unsigned long bity = *(unsigned long *)&cislo;

     &cislo -> získá adresu proměnné číslo
     (unsigned long *) -> přetypuje adresu na proměnnou typu float na ukazatel na ulong
     *adresa -> dereference, vrátí hodnotu odkazovanou danou adresou
     Výsledek má typ unsigned long, aniž by se provedlo oříznutí desetinné části.

     Poznámka: Překladač vypisuje varování, protože optimalizátor by to mohl
     potencionálně pokazit. Řešení s typem union se jeví jako mnohem bezpečnější,
     jenom je potřeba použít celočíselné typy správné velikosti. Typ float je
     podle normy 32 bitový, proto je typ uint32_t přesně to pravé. Pro typ
     double je vhodný typ uint64_t.

  */
}

typedef union
{
  double cislo;
  uint64_t bity;
} Tdoublehack;


void tiskniBityDouble(double cislo)
{
  Tdoublehack x;
  x.cislo = cislo;
//  unsigned long long bity = *(unsigned long long *)&cislo; // to je ale hnusný hack, co?
  uint64_t maska = 1ULL << (sizeof(double)*8-1); // 0000 0001 << delka-1

  // znaménkový bit
  putchar((x.bity & maska) ? '1' : '0');
  putchar('|');

  // exponent - v typu double má 11 bitů
  maska >>= 1;
  for (int i = 0; i < 11; ++i)
  {
    putchar((x.bity & maska) ? '1' : '0');
    maska >>= 1;
  }

  // mantisa - v typu double má 52 bitů
  putchar('|');
  for (int i = 0; i < 52; ++i)
  {
    putchar((x.bity & maska) ? '1' : '0');
    maska >>= 1;
  }
}
