/* Encyklopedie jazyka C
 *
 * Kapitola Číselné typy
 *
 */

/* Copyright (c) David Martinek, 2023
 * david.martinek.78 (zavináč) gmail.com
 *
 * This work is licensed under a Creative Commons
 * Attribution-NonCommercial-ShareAlike 4.0 International License.
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 *
 * Toto dílo je licencováno pole licence Creative Commons
 * Uveďte původ-Neužívejte dílo komerčně-Zachovejte licenci 4.0 Mezinárodní
 * (CC BY-NC-SA 4.0)
 * https://creativecommons.org/licenses/by-nc-sa/4.0/deed.cs
 */

#define VERZE "5.2"
#define NAZEV "Číselné typy"

#include "pomocny-modul.h"
#include "bitovy-tisk.h"

// Implementace nabídkového menu
#include "menu.h"

#include <stdio.h>
#include <stdlib.h>
// #include <string.h>
#include <stdbool.h>  // typ bool, konstanty true, false
#include <ctype.h>    // toupper, isalpha, ...
#include <math.h>     // INFINITY, NAN, sin, cos, log, exp, ...
#include <float.h>    // FLT_MIN, DBL_MAX, ...
#include <limits.h>   // SHRT_MIN, INT_MAX, ...
#include <complex.h>  // typ double complex,



// Tohle je tu pro případ, že překladač ve Windows tyto konstanty nezná.
#ifndef INFINITY
  #define INFINITY 1.0/0.0
#endif // INFINITY

#ifndef NAN
  #define NAN 0.0/0.0
#endif // NAN


// Tady jsou jen hlavičky funkcí. Implementace je za funkcí main.
void celociselneTypyPrehled(void);
void celociselneTypyFormatovanyTiskACteni(void);
void zapisCelociselnychKonstant(void);

void racionalniTypyPrehled(void);
void racionalniTypyFormatovanyTiskACteni(void);
void zapisRacionalnichKonstant(void);
void racionalniTypyProblemy(void);

void typChar(void);
void typCharFormatovanyTiskCteniOperace(void);

void logickyTypInt(void);
void logickyTypBool(void);

void aritmetickeOperatory(void);
void bitoveOperatory(void);
void pretypovaniVyrazu(void);

void komplexniTypyPrehled(void);
void komplexniTypyOperace(void);




/** Startovní bod programu. */
int main(void)
{
  povolBarevnyVystup();
//  prepniRychlostVypraveni(2);

  Tmenu * menu = menuVytvor(
    NADPIS(
      "==============================================================\n"
      "Encyklopedie jazyka C - " NAZEV " (verze " VERZE ")\n"
      "(c) David Martinek, 2023\n"
      "Zároveň s programem sleduj i zdrojový text!\n"
      "==============================================================\n"
    )
    , zobrazMod
  );

  if (menu == NULL)
  {
    printh("Není dost paměti ani pro výrobu menu. Končím program.");
    return 0;
  }


  menuPridej(menu,   0, "\n" SEKCE("Celočíselné typy") , NULL);
  menuPridej(menu, 'A', "Přehled\n", celociselneTypyPrehled);
  menuPridej(menu, 'B', "Formátovaný tisk a čtení\n", celociselneTypyFormatovanyTiskACteni);
  menuPridej(menu, 'C', "Zápis celočíselných konstant\n", zapisCelociselnychKonstant);

  menuPridej(menu,   0, "\n" SEKCE("Racionální typy") , NULL);
  menuPridej(menu, 'D', "Přehled\n", racionalniTypyPrehled);
  menuPridej(menu, 'E', "Formátovaný tisk a čtení\n", racionalniTypyFormatovanyTiskACteni);
  menuPridej(menu, 'F', "Zápis racionálních konstant\n", zapisRacionalnichKonstant);
  menuPridej(menu, 'G', "Na co si dát pozor\n", racionalniTypyProblemy);

  menuPridej(menu,   0, "\n" SEKCE("Typ char") , NULL);
  menuPridej(menu, 'H', "Číslo a znak\n", typChar);
  menuPridej(menu, 'I', "Formátovaný tisk a čtení + další operace\n", typCharFormatovanyTiskCteniOperace);

  menuPridej(menu,   0, "\n" SEKCE("Logické typy") , NULL);
  menuPridej(menu, 'J', "Typ int jako logický typ\n", logickyTypInt);
  menuPridej(menu, 'K', "Typ bool\n", logickyTypBool);

  menuPridej(menu,   0, "\n" SEKCE("Další operace s číselnými typy") , NULL);
  menuPridej(menu, 'L', "Aritmetické operátory\n", aritmetickeOperatory);
  menuPridej(menu, 'M', "Bitové operátory\n", bitoveOperatory);
  menuPridej(menu, 'N', "Přetypování výrazů\n", pretypovaniVyrazu);

  menuPridej(menu,   0, "\n" SEKCE("Komplexní čísla") , NULL);
  menuPridej(menu, 'O', "Přehled\n", komplexniTypyPrehled);
  menuPridej(menu, 'P', "Operace s komplexními čísly\n", komplexniTypyOperace);

  menuPridej(menu,   0, "\n" SEKCE("Nastavení programu"), NULL);
  menuPridej(menu, '0', "Nastav vyprávěcí mód\n", nastavRychlostVypraveni);

  while (menuVolba(menu) != 'Q') {}

  printg("Končím program.\n");
  menuZrus(menu);
  return 0;
}


////////////////////////////////////////////////////////////////////////////////
void celociselneTypyPrehled(void)
{
  zacatekKapitoly("Celočíselné typy - char, short, int, long, long long");

  printh(
    "- Slouží k ukládání celých čísel se znaménkem (" KK("signed") ") nebo bez (" KK("unsigned") ").\n"
    "- Rozsah je dán počtem bajtů, které zabírají v paměti.\n"
    "- Hodnoty mimo rozsah vedou k přetečení.\n"
    "- Funguje nad nimi celočíselná aritmetika.\n"
    "\n"
  );

  if (navratNeboSekce("Typ int")) return;
  if (pokracovat())
  {
    printh(
      "- Základní celočíselný typ\n"
  //  "- Navržen tak, aby měl velikost celočíselných registrů procesoru.\n"
      "- Dnes má typicky 32 bitů, tj. 4 bajty. Závisí to na nastavení překladače.\n"
      "- Počet bitů pro " KK("int") " v tomto programu: %zu bitů, tj. " KK("sizeof(int)*8") ".\n"
      "\n"
      , sizeof(int)*8
    );
  }

  if (navratNeboSekce("Velikosti celočíselných typů")) return;
  if (pokracovat())
  {
    printh(
      "- Na různých architekturách se mohou lišit!\n"
      "- Jedině typ " KK("char") " musí zabírat všude právě 1 bajt.\n"
      "- Neznaménkové typy zabírají stejný prostor.\n"
      "- Na všech architekturách ale musí platit toto:\n"
      "\n"
      KOD(
      "  sizeof(char)<=sizeof(short)<=sizeof(int)<=sizeof(long)<=sizeof(long long)\n"
      )
      "\n"
      "Skutečné velikosti typů v tomto programu (v bajtech):\n"
      "\n"
      KOD(
      "     sizeof(char): %zu B,      sizeof(unsigned char): %zu B  => %zu bitů\n"
      "    sizeof(short): %zu B,     sizeof(unsigned short): %zu B  => %zu bitů\n"
      "      sizeof(int): %zu B,       sizeof(unsigned int): %zu B  => %zu bitů\n"
      "     sizeof(long): %zu B,      sizeof(unsigned long): %zu B  => %zu bitů\n"
      "sizeof(long long): %zu B, sizeof(unsigned long long): %zu B  => %zu bitů\n"
      )
      "\n"
      , sizeof(char)      , sizeof(unsigned char)     , sizeof(char)*8
      , sizeof(short)     , sizeof(unsigned short)    , sizeof(short)*8
      , sizeof(int)       , sizeof(unsigned int)      , sizeof(int)*8
      , sizeof(long)      , sizeof(unsigned long)     , sizeof(long)*8
      , sizeof(long long) , sizeof(unsigned long long), sizeof(long long)*8
    );
  }

  if (navratNeboSekce("Typ size_t")) return;
  if (pokracovat())
  {
    printh(
      "- Celočíselný neznaménkový typ používaný pro udávání rozměrů paměti\n"
      "- Musí obsáhnout maximální alokovatelný rozměr paměti.\n"
      "- Používají ho např. " KK("sizeof") ", " KK("malloc") " aj.\n"
      "\n"
      KOD(
      "sizeof(size_t) = %zu B\n"
      )
      "\n"
      , sizeof(size_t)
    );
  }

  if (navratNeboSekce("Velikosti z limits.h")) return;
  if (pokracovat())
  {
    printh(
      "- Typ " KK("int") ", " KK("signed int") " - znaménkový typ pro kladná i záporná čísla\n"
      "  Minimální hodnota: " KK("INT_MIN = %d") "\n"
      "  Maximální hodnota: " KK("INT_MAX = %d") "\n"
      "\n"
      "- Typ " KK("unsigned int") " - neznaménkový typ, jen pro kladná čísla\n"
      "  Minimální hodnota: " KK("0") "\n"
      "  Maximální hodnota: " KK("UINT_MAX = %u") "\n"
      "\n"
      "- Ostatní celočíselné typy:\n"
      "\n"
      KOD(
      "   CHAR_MIN = %+hhd,                   SHRT_MIN = %+hd\n"
      "   CHAR_MAX = %+hhd,                   SHRT_MAX = %+hd\n"
      "  UCHAR_MAX = %+hhu,                   USHRT_MAX = %+hu\n"
      "\n"
      "   LONG_MIN = %+ld,  LLONG_MIN = %+lld\n"
      "   LONG_MAX = %+ld,  LLONG_MAX = %+lld\n"
      "  ULONG_MAX = %+lu, ULLONG_MAX = %+llu\n"
      )
      "\n"
      , INT_MIN, INT_MAX, UINT_MAX
      , CHAR_MIN,  SHRT_MIN
      , CHAR_MAX,  SHRT_MAX
      , UCHAR_MAX, USHRT_MAX
      , LONG_MIN,  LLONG_MIN
      , LONG_MAX,  LLONG_MAX
      , ULONG_MAX, ULLONG_MAX
    );

    /* Typ int je základní celočíselný typ pro běžné celočíselné výpočty.

    Je navržen tak, aby práce s ním byla co nejefektivnější => jeho velikost by
    měla odpovídat velikosti registru procesoru. Na 32b architektuře by měl mít
    velikost 32 bitů, tedy 4 bajty. Na 64b architektuře by to mělo být 8 bajtů,
    ale pravděpodobně s ohledem na snazší přechod od 32b na 64b architekturu se
    výrobci procesorů s výrobci překladačů dohodli, že i na 64b procesorech bude
    mít typ int 32 bitů. Velikost 64 bitů tedy má až typ long (na MS Windows ne,
    viz dále).

    Jak to skutečně je v daném programu, záleží také na nastavení překladače.
    Překladač umí překládat i pro jiné architektury, než na jaké aktuálně běží.
    Dává to smysl při tvorbě programů pro starší operační systémy. Na 64b
    operačním systému obvykle můžou běžet i 32b programy (opačně to ale nejde).

    Poznámka k MS Windows
    Microsoft požaduje, aby i na 64b architektuře měl typ long 32 bitů, takže
    se toho drží i překladače. I když máte 64b program, budete tedy mít
    k dispozici jen 32b typ long. MS musí mít ve všem nějaké extra buřty...

    sizeof(int)
    - operátor sizeof vrací počet bajtů, které zabírá hodnota daného typu
    - vrací hodnotu typu size_t, což je celočíselný neznaménkový typ
    - hodnota tohoto typu se tiskne značkou %zu

    Velikost tohoto typu (jakož i ostatních celočíselných typů) může být na
    různých platformách různá. Pokud se program spoléhá na jeho velikost, měl
    by si ji nejprve ověřit pomocí operátoru sizeof nebo pomocí konstant
    z limits.h.

    signed
    - znaménkový typ - výchozí, většinou se neuvádí

    unsigned
    - neznaménkový (bez záporných čísel)
    - Nemění velikost typu: sizeof(int) == sizeof(unsigned int).
    - Nemění rozsah, obě varianty dokáží vyjádřit stejný počet hodnot, jenom mají
    posunuty hodnoty minima a maxima. Typ unsigned int má dvojnásobný rozsah
    v kladném směru. Typ (signed) int má nejvyšší kladnou hodnotu poloviční, ale
    zase má záporné hodnoty.

    Předpony signed a unsigned říkají, jak se bude interpretovat nejvyšší bit
    čísla. U signed se vyhodnotí jako znaménko (0 - kladné, 1 - záporné), u typu
    unsigned se tento bit považuje za součást (kladné) hodnoty, i když je to 1.

    Znaménkové typy se ukládají v doplňkovém kódu.
    */
  }

  if (navratNeboSekce("Typy s pevnou velikostí")) return;
  if (pokracovat())
  {
    #include <stdint.h>

    printh(
      "- Pokud je z nějakého důvodu potřeba zajistit pevnou velikost číselných\n"
      "  typů, lze použít typy z modulu " KK("stdint.h") ".\n"
      "- U těchto typů nejde použít předpony " KK("signed") " a " KK("unsigned") ".\n"
      "- Místo toho jsou přímo definovány znaménkové a neznaménkové varianty.\n"
      "\n"
      KOD(
      "   sizeof(int8_t): %zu B,  sizeof(uint8_t): %zu B  => %zu bitů\n"
      "  sizeof(int16_t): %zu B, sizeof(uint16_t): %zu B  => %zu bitů\n"
      "  sizeof(int32_t): %zu B, sizeof(uint32_t): %zu B  => %zu bitů\n"
      "  sizeof(int64_t): %zu B, sizeof(uint64_t): %zu B  => %zu bitů\n"
      )
      "\n"
      , sizeof(int8_t) , sizeof(uint8_t) , sizeof(int8_t) *8
      , sizeof(int16_t), sizeof(uint16_t), sizeof(int16_t)*8
      , sizeof(int32_t), sizeof(uint32_t), sizeof(int32_t)*8
      , sizeof(int64_t), sizeof(uint64_t), sizeof(int64_t)*8
    );

    printh(
      "- Také existují typy " KK("int_leastXX_t") " a " KK("uint_leastXX_t") ", kde XX je 8--64.\n"
      "  => Mají velikost nejméně XX bitů, ale můžou mít i více.\n"
      "\n"
      "- Dále existují typy " KK("int_fastXX_t") " a " KK("uint_fastXX_t") ", kde XX je 8--64.\n"
      "  => Mají velikost alespoň XX bitů, ale ve skutečnosti mají šířku, která\n"
      "     je na dané architektuře nejefektivnější.\n"
      "  => Je to kompromis mezi efektivitou a zaručeným rozsahem.\n"
      "\n"
    );
  }


  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void celociselneTypyFormatovanyTiskACteni(void)
{
  zacatekKapitoly("Celočíselné typy - formátovaný tisk a čtení");

  printh(
    "- Slouží pro to funkce ze stdio.h.\n"
    "- Formátovaný tisk se dělá pomocí funkcí " KK("printf") ", " KK("fprintf") ", ...\n"
    "- Formátovaný vstup se dělá pomocí funkcí " KK("scanf") ", " KK("fscanf") ", ...\n"
    "\n"
  );

  if(navratNeboSekce("Správný překlad ve Windows")) return;
  if (pokracovat())
  {
    printh(
      "- Na začátku programu, před ostatními příkazy " KK("#include") " použij:\n"
      "\n"
      KOD(
      "  #define __USE_MINGW_ANSI_STDIO 1\n"
      )
      "\n"
      "- Bez tohoto nebudou fungovat některé zde vyjmenované formátovací značky.\n"
  //    "- Standardně totiž překladač používá knihovnu jazyka C od firmy Microsoft,\n"
  //    "  která tyto vlastnosti jazyka C99 neumí používat!\n"
  //    "- Definování této konstanty přinutí překladač použít správnou verzi\n"
  //    "  knihovny.\n"
      "- V Linuxu to není potřeba, C99 tady funguje bez problémů.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Formátové značky pro printf a scanf")) return;
  if (pokracovat())
  {
    printh(
      "- Obě funkce používají pro celá čísla stejné základní formátové značky.\n"
      "\n"
      KOD(
      " signed dekadicky |    unsigned dekadicky    |      unsigned hexa\n"
      "       char: %%hhd |      unsigned char: %%hhu |      unsigned char: %%hhx\n"
      "      short:  %%hd |     unsigned short:  %%hu |     unsigned short:  %%hx\n"
      "        int:   %%d |       unsigned int:   %%u |       unsigned int:   %%x\n"
      "       long:  %%ld |      unsigned long:  %%lu |      unsigned long:  %%lx\n"
      "  long long: %%lld | unsigned long long: %%llu | unsigned long long: %%llx\n"
      )
      "\n"
      "- Typ " KK("size_t: %%zu") "\n"
      "\n"
      "- Poznámka k hexa zobrazení: x tiskne malé, X velké číslice (1af vs. 1AF).\n"
      "\n"
    );
  }

  if (navratNeboSekce("Obecný formát značek pro printf")) return;
  if (pokracovat())
  {
    printh(
      "- Funkce " KK("printf") " přidává ke značkám modifikátory pro formátovaný tisk.\n"
      "\n"
      KOD(
      "  %%<modifikátor><značka>\n"
      )
      "\n"
      "  <modifikátor> - není povinný, ovlivňuje zarovnávání a formátování\n"
      "  <značka> - povinná, určuje typ následujícího argumentu, který se bude tisknout\n"
      "\n"
      PODSEKCE("Modifikátory")
      "  5  -> %%5hd   -> zarovnej hodnotu typu " KK("short") " na 5 míst doprava mezerami\n"
      "  05 -> %%05hhd -> zarovnej číslo " KK("char") " na 5 míst doprava nulami\n"
      "  -5 -> %%-5d   -> zarovnej " KK("int") " na 5 míst doleva mezerami\n"
      "  *   -> %%*ld    -> zarovnej " KK("long") " doprava mezerami na šířku danou argumentem\n"
      "\n"
    );

    unsigned int a = 123; int b = -321;
    // tisk bez zarovnání
    printh(
      KOD(
      "  unsigned int a = 123; int b = -321;\n"
      "  printf(\"a=%%u, a=%%X, b=%%d\", a, a, b);"
      )
      "              => a=%d, a=%X, b=%d\n"
      , a, a, b
    );

    // zarovnej doprava na šířku 5 znaků
    printh(
      KOD(
      "  printf(\"a=%%5u, a=%%5X, b=%%5d\", a, a, b);"
      )
      "           => a=%5d, a=%5X, b=%5d\n"
      , a, a, b
    );

    // zarovnej doprava nulami na šířku 5 znaků
    printh(
      KOD(
      "  printf(\"a=%%05u, a=%%05X, b=%%05d\", a, a, b);"
      )
      "        => a=%05d, a=%05X, b=%05d\n"
      , a, a, b
    );

    // zarovnej doleva na šířku 5 znaků
    printh(
      KOD(
      "  printf(\"a=%%-5u, a=%%-5X, b=%%-5d\", a, a, b);"
      )
      "        => a=%-5d, a=%-5X, b=%-5d\n"
      , a, a, b
    );

    // šířka na místě * je dána parametrem -> %*d očekává dva parametry -> zde 5, a
    printh(
      KOD(
      "  printf(\"a=%%*u, a=%%*X, b=%%*d\", 5, a, 5, a, 5, b);"
      )
      "  => a=%*d, a=%*X, b=%*d\n\n"
      , 5, a, 5, a, 5, b
    );
  }

  if (navratNeboSekce("Formát značek pro scanf")) return;
  if (pokracovat())
  {
    printh(
      "- Odpovídá základnímu tvaru značek pro " KK("printf") ".\n"
      "- Nemá modifikátory.\n"
      "- Mezi značkami jde předepsat oddělovače.\n"
      "  - %%d%%d    -> oddělovačem jsou bílé znaky\n"
      "  - %%d,%%d   -> oddělovačem je čárka\n"
      "  - %%d , %%d -> oddělovačem je čárka a libovolný počet bílých znaků\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka čtení - %%d%%d")) return;
  if (pokracovat())
  {
    printh(
      "- Jako oddělovač jde zadat libovolně dlouhou posloupnost bílých znaků.\n"
      "\n"
      KOD(
      "  int c, d, k;\n"
      "  k = scanf(\"%%d%%d\", &c, &d);\n"
      )
      "\n"
    );
    char znovu = 'z';
    while (znovu == 'z')
    {
      printg(
        "  Zadej dvě celá čísla (kladné a záporné) oddělená mezerami:\n"
        "  "
      );

      int c, d, k;
      // mezi dvě značky %d nemusím psát nic, přesto scanf automaticky přeskočí bílé znaky
      k = scanf("%d%d", &c, &d);
      // Funkční (návratová) hodnota funkce scanf udává počet úspěšně zpracovaných značek.
      pozerKonecRadku(); // Odstraní ze vstupu řádek, kdyby uživatel zadal něco špatně.
      printh(
        "\n"
        "  => c:%d, d:%d, k:%d (k=2 je ok, jiná hodnota je chyba)\n"
        "\n"
        , c, d, k
      );

      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  if (navratNeboSekce("Ukázka čtení %%d,%%d")) return;
  if (pokracovat())
  {
    printh(
      "- Oddělovačem je čárka. Kolem ní nesmí být bílé znaky.\n"
      "\n"
      KOD(
      "  int c, d, k;\n"
      "  k = scanf(\"%%d,%%d\", &c, &d);\n"
      )
      "\n"
    );
    /* Když do formátového řetězce přidám oddělovač, bude ho tam scanf očekávat.
       Když teď uživatel nějaké mezery zadá, vyhodnotí se to jako chyba.
    */
    char znovu = 'z';
    while (znovu == 'z')
    {
      printg(
        "  Zadej dvě celá čísla oddělená čárkou:\n"
        "  "
      );
      int c, d, k;
      k = scanf("%d,%d", &c, &d);
      pozerKonecRadku(); // Odstraní ze vstupu řádek, kdyby uživatel zadal něco špatně.
      printh(
        "\n"
        "  => c:%d, d:%d, k:%d (k=2 je ok, jiná hodnota je chyba)\n"
        "\n"
        , c, d, k
      );
      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  if (navratNeboSekce("Ukázka čtení %%d , %%d")) return;
  if (pokracovat())
  {
    printh(
      "- Oddělovačem je čárka. Kolem ní mohou být bílé znaky.\n"
      "\n"
      KOD(
      "  int c, d, k;\n"
      "  k = scanf(\"%%d , %%d\", &c, &d);\n"
      )
      "\n"
    );
    /* Když do formátového řetězce přidám oddělovač, bude ho tam scanf očekávat.
       Mezery zde umožní přeskočit možné bílé znaky mezi hodnotami a oddělovačem.
    */
    char znovu = 'z';
    while (znovu == 'z')
    {
      printg(
        "  Zadej dvě celá čísla oddělená čárkou:\n"
        "  "
      );
      int c, d, k;
      k = scanf("%d , %d", &c, &d);
      pozerKonecRadku(); // Odstraní ze vstupu řádek, kdyby uživatel zadal něco špatně.
      printh(
        "\n"
        "  => c:%d, d:%d, k:%d (k=2 je ok, jiná hodnota je chyba)\n"
        "\n"
        , c, d, k
      );
      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  if (navratNeboSekce("Pozor na značku %%u")) return;
  if (pokracovat())
  {
    /* Pozor! Čtení značkou %u uživateli nezabrání v zadání záporné hodnoty.
       Ani nejde otestovat, že ji zadal. Zadá-li uživatel třeba hodnotu -10, dojde
       k přetečení a v cílové proměnné se ocitne hodnota UINT_MAX - 10 + 1.
    */
    printh(
      "- Pozor při čtení neznaménkových čísel značkou %%u.\n"
      "- Tato značka nezabrání uživateli zadat zápornou hodnotu. => Vede to k přetečení.\n"
      "\n"
      KOD(
      "  unsigned int e, f;\n"
      "  int k;\n"
      "  k = scanf(\"%%u%%u\", &e, &f);\n"
      )
      "\n"
    );
    char znovu = 'z';
    while (znovu == 'z')
    {
      printg(
        "  Zadej dvě celá čísla (kladné a záporné) oddělená mezerou:\n"
        "  "
      );
      unsigned int e, f;
      int k;
      k = scanf("%u%u", &e, &f);
      pozerKonecRadku(); // Odstraní ze vstupu řádek, kdyby uživatel zadal něco špatně.
      printh(
        "\n"
        "  => e:%u, f:%u, k:%d (k=2 je ok, jiná hodnota je chyba)\n"
        "\n"
        "- Při zadání záporného čísla dojde k přetečení.\n"
        "\n"
        , e, f, k
      );
      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void zapisCelociselnychKonstant(void)
{
  zacatekKapitoly("Zápis celočíselných konstant - literálů");

  printh(
    "- Literál je přímý zápis konstantní hodnoty.\n"
    "- Používá se, když je ve zdrojovém textu potřeba přímo zapsat hodnotu.\n"
    "\n"
    "- celočíselné typy: " KK("1") ", " KK("-78") ", ...\n"
    "- racionální typy: " KK("12.3") ", " KK("14e5") ", ...\n"
    "- typ znak: " KK("'a'") ", " KK("'Z'") ", " KK("'\\n'") ", ... - Ve skutečnosti jsou typu " KK("int") "!\n"
//    "- textové řetězce: " KK("\"Ahoj světe!\"") "\n"
    // Není to celočíselná kontanta. O stringových literálech je poznámka
    // v kapitole Stringy.
    "\n"
  );

  if (navratNeboSekce("Dlouhé typy int, long, long long")) return;
  if (pokracovat())
  {
    printh(
      PODSEKCE("Typ int")
      "\n"
      KOD(
      "  %d, %+d, %+d\n"
      )
      "\n"
      // int - bez přípony
      , 1234567890, -1234567890, +1234567890
    );
    printh(
      PODSEKCE("Typ long")
      "- přípona l, L - raději velké, nezamění se l za 1\n"
      "\n"
      KOD(
      "  %ldL, %+ldL, %+ldL\n" // značka je %ld, další L je přípona tištěná na výstup
      )
      "\n"
      // přípona L - long int
      , 1234567890L, -1234567890L, +1234567890L
    );
    printh(
      PODSEKCE("Typ long long")
      "- přípona ll, LL - raději velké, nezamění se ll za 11\n"
      "\n"
      KOD(
      "  %lldLL, %+lldLL, %+lldLL\n" // značka je %lld, další LL je přípona tištěná na výstup
      )
      "\n"
      // přípona LL - long long int
      , 1234567890123456789LL, -1234567890123456789LL, +1234567890123456789LL
    );
  }

  if (navratNeboSekce("Kratší typy char a short")) return;
  if (pokracovat())
  {
    #define VELKE 123456 // varování překladače - přetečení konstanty
    char c = VELKE;      // varování překladače - přetečení konstanty
    printh(
      "- Žádnou příponu nemají.\n"
      "- Konstanty jsou typu " KK("int") ".\n"
      "- Větší hodnota může přetéct.\n"
      "\n"
      KOD(
      "  char c = %d;\n"
      )
      "\n"
      "  => c: %hhd\n"
      "\n"
      , VELKE, c
    );
  }

  if (navratNeboSekce("Typy unsigned")) return;
  if (pokracovat())
  {
    printh(
      "- Přípona u, ul, ull, U, UL, ULL\n"
      "\n"
      KOD(
      "        unsigned int: %uU\n"    // značka je %u, další U je přípona tištěná na výstup
      "       unsigned long: %luUL\n"
      "  unsigned long long: %lluULL\n"
      )
      "\n"
      , 1234567890U, 1234567890UL, 1234567890123456789ULL
    );
  }

  if (navratNeboSekce("Zápis v jiných číselných soustavách")) return;
  if (pokracovat())
  {
    printh(
      "- Osmičková (octal) soustava -> předpona 0 (nula)\n"
      "  osmičkově: 0%o\n"
      "  desítkově: %d\n"
      "  Osmičkový tisk pomocí 0%%o je unsigned: -0%o => 0%o\n" // značka %o tiskne na výstup v osmičkové soustavě jako unsigned!
      "  Pozor! Netiskne to předponu 0. Tu tisknu ručně.\n"
      "\n"
      , 01234567, 01234567, 01234567, -01234567
    );
    printh(
      "- Šestnáctková (hex) soustava -> předpona 0x, 0X (nula x, X)\n"
      "  šestnáctkově: 0x%X\n"
      "  desítkově:    %d\n"
      "  Hexa tisk pomocí 0x%%X je unsigned: -0x%X => 0x%X\n" // značka je %x tiskne na výstup v šestnáctkové soustavě jako unsigned
      "  Pozor! Netiskne to předponu 0x. Tu tisknu ručně.\n"
      "\n"
      , 0x1234CDEF, 0x1234CDEF, 0x1234CDEF, -0x1234CDEF
    );

    printh(
      "- Dvojková soustava -> předpona 0b, 0B (nula b, B)\n"
      "- Pozor! Zatím jen rozšíření GCC - možná bude ve standardu C23.\n"
  //    "  %B, -%B => %B\n"     -- tohle nefunguje
      "  dvojkově: 0b10101, -0b10101\n"
      "  desítkově: %d, %d\n"
      "\n"
      , 0b10101, -0b10101  // Varování, že binární konstanty jsou rozšířením GCC.
    );
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void racionalniTypyPrehled(void)
{
  zacatekKapitoly("Racionální typy - float, double, long double");

  printh(
    "- Jde o typy pro práci s desetinnými čísly.\n"
    "- Reálná čísla nejde obecně na počítači uložit přesně => proto racionální.\n"
    "- Čísla mohou mít (relativně velký) exponent, ale jen omezený desetinný rozvoj.\n"
    "\n"
    SEKCE("Velikost typů v bajtech")
    "\n"
    KOD(
    "  sizeof(float): %zu, sizeof(double): %zu, sizeof(long double): %zu\n"
    )
    "\n"
    , sizeof(float)
    , sizeof(double)
    , sizeof(long double)
  );

  if (navratNeboSekce("Konstanty z math.h (ISO C99)")) return;
  if (pokracovat())
  {
    printh(
      "- Jde pracovat i s nekonečnem - třeba výsledek po dělení nulou 1.0/0.0.\n"
      "\n"
      KOD(
      "  INFINITY = %f, -INFINITY = %f\n"
      )
      "\n"
      "- V Linuxu se tiskne text " KK("inf") ", " KK("-inf") ". Na vstupu tomu rozumí i " KK("scanf") ".\n"
      "- Ve Windows to nefunguje s výchozí implementací od Microsoftu.\n"
      "  Je potřeba zapnout verzi knihovny z projektu MinGW pomocí\n"
      "  konstrukce " KK("#define __USE_MINGW_ANSI_STDIO 1") ".\n"
      "  Viz kapitolu Celočíselné typy - formátovaný tisk a čtení.\n"
      "\n"
      "- Hodnota " KK("NAN") " znamená nedefinovanou hodnotu - třeba výsledek výrazu " KK("0.0/0.0") ",\n"
      "  " KK("NAN") " -- Not a Number.\n"
      "\n"
      KOD(
      "  NAN = %f\n"
      )
      "  (V Linuxu se tiskne text " KK("nan") ". Na vstupu tomu rozumí i " KK("scanf") ".)\n"
      "\n"
      // Konstanty INFINITY a NAN zavádí C99 v math.h. V některých implementacích
      // překladače ale chybí.
      , INFINITY  // 1.0/0.0 - tohle jde použít, když konstanta v math.h chybí
      , -INFINITY
      , NAN       // 0.0/0.0 - tohle jde použít, když konstanta v math.h chybí
    );
  }

  if (navratNeboSekce("Velikosti z float.h")) return;
  if (pokracovat())
  {
    printh(
      "- Přibližný počet desítkových desetinných míst v mantise\n"
      KOD(
      "    FLT_DIG = %d, DBL_DIG = %d, LDBL_DIG = %d\n"
      )
      "\n"
      PODSEKCE("Typ float")
      KOD(
      "  FLT_MIN = %.*e, FLT_MAX = %.*e\n"
      )
      "\n"
      "- Hodnota exponentu (skutečný dvojkový)\n"
      KOD(
      "  FLT_MIN_EXP = %d, FLT_MAX_EXP = %d\n"
      )
      "\n"
      "- Hodnota exponentu (desítkový)\n"
      KOD(
      "  FLT_MIN_10_EXP = %d, FLT_MAX_10_EXP = %d\n"
      )
      "\n"
      PODSEKCE("Typ double")
      "- Minimum a maximum\n"
      KOD(
      "  DBL_MIN = %.*e, DBL_MAX = %.*e\n"
      )
      "\n"
      "- Hodnota exponentu\n"
      KOD(
      "  DBL_MIN_EXP = %d, DBL_MAX_EXP = %d\n"
      )
      "\n"
      "- Hodnota exponentu\n"
      KOD(
      "  DBL_MIN_10_EXP = %d, DBL_MAX_10_EXP = %d\n"
      )
      "\n"
      PODSEKCE("Typ long double")
      "- Minimum a maximum\n"
      KOD(
      "  LDBL_MIN = %.*Le, LDBL_MAX = %.*Le\n"
      )
      "\n"
      "- Hodnota exponentu (skutečný dvojkový)\n"
      KOD(
      "  LDBL_MIN_EXP = %d, LDBL_MAX_EXP = %d\n"
      )
      "\n"
      "- Hodnota exponentu (desítkový)\n"
      KOD(
      "  LDBL_MIN_10_EXP = %d, LDBL_MAX_10_EXP = %d\n"
      )
      "\n"
      , FLT_DIG, DBL_DIG, LDBL_DIG

      , FLT_DIG, FLT_MIN, FLT_DIG, FLT_MAX
      , FLT_MIN_EXP, FLT_MAX_EXP
      , FLT_MIN_10_EXP, FLT_MAX_10_EXP

      , DBL_DIG, DBL_MIN, DBL_DIG, DBL_MAX
      , DBL_MIN_EXP, DBL_MAX_EXP
      , DBL_MIN_10_EXP, DBL_MAX_10_EXP

      , LDBL_DIG, LDBL_MIN, LDBL_DIG, LDBL_MAX
      , LDBL_MIN_EXP, LDBL_MAX_EXP
      , LDBL_MIN_10_EXP, LDBL_MAX_10_EXP
      );
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void racionalniTypyFormatovanyTiskACteni(void)
{
  zacatekKapitoly("Racionální typy - formátovaný tisk a čtení");

  printh(
    SEKCE("Formátovací značky pro printf")
    "\n"
    KOD(
    "  float a double:  %%f  %%e  %%g\n"
    "     long double: %%Lf %%Le %%Lg\n"
    )
    "\n"
    SEKCE("Formátové značky pro scanf")
    "\n"
    KOD(
    "        float:  %%f\n"
    "       double: %%lf\n"
    "  long double: %%Lf\n"
    )
    "\n"
  );

  if (navratNeboSekce("Formát značek racionálních typů pro printf")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  %%<modifikátor><značka>\n"
      )
      "\n"
      "  <modifikátor> - není povinný, ovlivňuje zarovnávání a formátování\n"
      "  <značka> - povinná, určuje typ následujícího argumentu, který se bude tisknout\n"
      "\n"
      "  Modifikátory\n"
      "    10   -> %%10f   -> zarovnej desetinné číslo na 10 míst doprava mezerami\n"
      "    010  -> %%010f  -> zarovnej desetinné číslo na 10 míst doprava nulami\n"
      "    -10  -> %%-10f  -> zarovnej desetinné číslo na 10 míst doleva mezerami\n"
      "    *    -> %%*f    -> zarovnej desetinné číslo doprava mezerami na šířku danou argumentem\n"
      "    .2   -> %%.2f   -> tiskni desetinné číslo zaokrouhlené na 2 desetinná místa\n"
      "    .*   -> %%.*f   -> zaokrouhli desetinné číslo na počet pozic daný argumentem\n"
      "    10.2 -> %%10.2f -> zarovnej na 10 míst a zaokrouhli na 2 desetinná místa\n"
      "    *.*  -> %%*.*f  -> zarovnej a zaokrouhli na počty dané 2 argumenty\n"
      "\n"
    );
  }

  if (navratNeboSekce("Použití základních formátovacích značek pro printf")) return;
  if (pokracovat())
  {
    /* Formátovací značky pro printf:

    %f - Tiskne hodnotu v základním tvaru bez exponentu.
         => U velkých čísel může být výsledek hodně dlouhý.
         => U malých čísel může tisknout nulu.

    %e - Tiskne hodnotu v exponenciálním tvaru.

    %g - V podstatě pro danou hodnotu vybere možnost %f nebo %g tak, aby výsledek
         zabíral na obrazovce co nejméně místa.

    %f, %e, %g - pro tisk hodnot typů float a double
    %Lf, %Le, %Lg - pro tisk hodnot typu long double (předpona L)
    */
    float a = 123.f, b = -123.f, c = 123.456e-12f;
    long double x = 123.L, y = -123.L, z = 123.456e400L;
    printh(
      "- Typy " KK("float") " a " KK("double") ": %%f, %%e, %%g.\n"
      "\n"
      KOD(
      "  float a = 123.f, b = -123.f, c = 123.456e-23f;\n"
      )
      "\n"
      "  a: %%f => %f, %%e => %e, %%g => %g\n"
      "  b: %%f => %f, %%e => %e, %%g => %g\n"
      "  c: %%f => %f, %%e => %e, %%g => %g\n"
      "\n"
      "- Typ " KK("long double") ": %%Lf, %%Le, %%Lg.\n"
      "\n"
      KOD(
      "  long double x = 123.L, y = -123.L, z = 123.456e400L;\n"
      )
      "\n"
      "  x: %%Lf => %Lf, %%Le => %Le, %%Lg => %Lg\n"
      "  y: %%Lf => %Lf, %%Le => %Le, %%Lg => %Lg\n"
      "  z: %%Lf => %Lf, %%Le => %Le, %%Lg => %Lg\n"
      "\n"
      ,a,a,a, b,b,b, c,c,c
      ,x,x,x, y,y,y, z,z,z
    );
  }

  if (navratNeboSekce("Použití modifikátorů značek pro printf")) return;
  if (pokracovat())
  {
    /* Modifikátory značek pro printf

    %.2f - zaokrouhlení na 2 desetinná místa

    %10f - zarovnej doprava na 10 pozic (včetně znaků . + e), případně doplň mezerami

    %-10f - zarovnej doleva na 10 pozic (včetně znaků . + e), případně doplň mezerami

    %10.2f - zaokrouhli na 2 desetinná místa a zarovnej doprava na 10 pozic
            (včetně znaků . + e), případně doplň mezerami

    %*.*f - délka zarovnání i zaokrouhlení je dána dalšími argumenty před vlastní hodnotou

    Stejně jako značku %f jde modifikovat i značky %e a %g.
    Totéž funguje i pro tisk hodnot typu long double %Lf, %Le a %Lg.
    */
    float a = 123.0f;
    printh(
      "\n"
      KOD(
      "  float a = 123.0f;\n"
      )
      "\n"
      "- Zaokrouhlení na 2 desetinná místa: %%.2f\n"
      "    a: |%%.2f| => |%.2f|\n"
      "       |%%.2e| => |%.2e|\n"
      "       |%%.2g| => |%.2g|\n"
      "- Zarovnání doprava mezerami na šířku 10 znaků: %%10f\n"
      "    a: |%%10f| => |%10f|\n"
      "       |%%10e| => |%10e|\n"
      "       |%%10g| => |%10g|\n"
      "- Zarovnání doleva mezerami na šířku 10 znaků: %%-10f\n"
      "    a: |%%-10f| => |%-10f|\n"
      "       |%%-10e| => |%-10e|\n"
      "       |%%-10g| => |%-10g|\n"
      "- Zarovnání na desetinnou tečku\n"
      "  - zaokrouhleno na 2 místa a tisk na šířku 20 znaků: %%20.2f\n"
      "    a: |%%20.2f| => |%20.2f|\n"
      "       |%%20.2e| => |%20.2e|\n"
      "       |%%20.2g| => |%20.2g|\n"
      "\n"
      ,a,a,a
      ,a,a,a
      ,a,a,a
      ,a,a,a
    );
    // TODO: vyzkoušejte si i další varianty a kombinace

    printh(
      "- Zarovnání i zaokrouhlení dáno parametricky\n"
      "\n"
      KOD(
      "  printf(\"|%%*.*f|\", 10, 3, a)\n"
      )
      "\n"
      "  =>|%*.*f|\n"
      "\n"
      , 10, 3, a
    );
  }

  if (navratNeboSekce("Formát značek racionálních typů pro scanf")) return;
  if (pokracovat())
  {
    /* Formátové značky pro scanf

    Při čtení se MUSÍ pracovat s typy proměnných PŘESNĚ odpovídajícím značkám.
    Pokud se například místo očekávané proměnné typu float dosadí proměnná typu
    double, zapíše se do ní nesmysl -- protože mají jinak velké mantisy
    a exponenty, jednotlivé bity se zapíší na špatná místa (nebude to lícovat).
    Když se větší typ zapíše do menší proměnné, dojde k zápisu mimo prostor této
    proměnné, což může přepsat vedle ležící lokální proměnnou. Projevuje se to
    zdánlivě záhadnými chybami, kdy se v proměnných objevují nesmyslné hodnoty
    zdánlivě bez příčiny.

    %f - čti hodnotu do proměnné typu float -> musí se odkazem předat proměnná
         typu float

    %lf - čti hodnotu do proměnné typu double -> musí se odkazem předat proměnná
          typu double

    %Lf - čti hodnotu do proměnné typu long double -> musí se odkazem předat
          proměnná typu long double
    */

    printh(
      "- Pozor na rozdíl " KK("float") " (%%f), " KK("double") " (%%lf) a " KK("long double") " (%%Lf)!\n"
      "- Při čtení se nepoužívají modifikátory značek.\n"
      "\n"
      "- Mezi značkami jde předepsat oddělovače.\n"
      "- Ukázky oddělovačů viz sekci Formátované čtení celočíselných typů.\n"
      "\n"
      "- Na vstupu mohou být\n"
      "  - hodnoty i v exponenciálním tvaru (1.5e-10),\n"
      "  - desetinná čísla s desetinnou tečkou, ne čárkou,\n"
      "  - hodnoty nan, inf, -inf,\n"
      "  - hodnoty s vynechanou celou nebo desetinnou částí (1, .5, 1e5, .1e3).\n"
      "    (Samotná tečka ale není číslo.)\n"
      "\n"
    );
  }

  if (navratNeboSekce("Ukázka čtení značkou %%f")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  float fltCislo;\n"
      "  int kod = scanf(\"%%f\", &fltCislo);\n"
      )
      "\n"
    );
    char znovu = 'z';
    while (znovu == 'z')
    {
      printg(
        "  Zadej desetinné číslo velikosti float (značka %%f):\n"
        "  "
      );

      float fltCislo;
      int kod = scanf("%f", &fltCislo);
      pozerKonecRadku(); // Kdyby uživatel zadal hodnotu chybně.
      printh(
        "\n"
        "  => fltCislo: %g, kod: %d (kod=1 je ok, něco jiného je chyba)\n"
        "\n"
        , fltCislo, kod
      );

      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  if (navratNeboSekce("Ukázka čtení značkou %%lf")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  double dblCislo;\n"
      "  int kod = scanf(\"%%lf\", &dblCislo);\n"
      )
      "\n"
    );

    char znovu = 'z';
    while (znovu == 'z')
    {
      printg(
        "  Zadej desetinné číslo velikosti double (značka %%lf):\n"
        "  "
      );

      double dblCislo;
      int kod = scanf("%lf", &dblCislo);
      pozerKonecRadku(); // Kdyby uživatel zadal hodnotu chybně.
      printh(
        "\n"
        "  => dblCislo: %g, kod: %d (kod=1 je ok, něco jiného je chyba)\n"
        "\n"
        , dblCislo, kod
      );

      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  if (navratNeboSekce("Ukázka čtení značkou %%Lf")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  long double ldblCislo;\n"
      "  int kod = scanf(\"%%Lf\", &ldblCislo);\n"
      )
      "\n"
    );

    char znovu = 'z';
    while (znovu == 'z')
    {
      printg(
        "  Zadej desetinné číslo velikosti long double (značka %%Lf):\n"
        "  "
      );

      long double ldblCislo;
      int kod = scanf("%Lf", &ldblCislo);
      pozerKonecRadku(); // Kdyby uživatel zadal hodnotu chybně.
      printh(
        "\n"
        "  => ldblCislo: %Lg, kod: %d (kod=1 je ok, něco jiného je chyba)\n"
        , ldblCislo, kod
      );

      printg(VOLBA("## Znovu? -> Z, Pokračuj -> Enter: "));
      znovu = tolower(prvniPismenoNaRadku());
    }
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void zapisRacionalnichKonstant(void)
{
  zacatekKapitoly("Zápis racionálních konstant - literálů");

  /* Racionální čísla - Protože na počítači nejde vyjádřit všechna reálná čísla,
     je korektní mluvit o desetinných číslech na počítači jako o racionálních
     číslech (takové, co jdou vyjádřit zlomkem). Iracionální čísla jako je třeba
     Pi nebo E se nutně ukládají zaokrouhlené (s useknutým desetinným rozvojem).

     Literál - Tímto pojmem se označuje přímý zápis konstantní hodnoty. Literály
     mohou být racionálních typů (12.3, 14e5 atd.), celočíselných typů (1, -78
     atd.), znakové ('a', 'Z', '\n' atd.) a řetězcové ("Ahoj světe!").
  */

  printh(
    SEKCE("Racionální konstanty obecně")
    "- Obsahují desetinnou tečku nebo exponent.\n"
    "\n"
    KOD(
    "  0.123, 12e34\n"
    )
    "\n"
    "- Jde vynechat celou část, nebo desetinnou část, ale ne obě.\n"
    "\n"
    KOD(
    "  .123 == 0.123, 123. == 123.0\n"
    )
    "\n"
    "- Samotná tečka je chyba.\n"
    "\n"
    "- Mohou být záporné.\n"
    "\n"
    KOD(
    "  -.123 == -0.123\n"
    )
    "\n"
    "- Mohou mít záporný exponent.\n"
    "\n"
    KOD(
    "  123e-5 == 0.00123\n"
    )
    "\n"
    "- Exponent může být e nebo E.\n"
    "\n"
    KOD(
    "  123e6 == 123E6 == 123000000.0\n"
    )
    "\n"
  );

  if (navratNeboSekce("Zápis racionálních literálů")) return;
  if (pokracovat())
  {
    printh(
      PODSEKCE("Typ float")
      "- S příponou f, F\n"
      "\n"
      KOD(
      "  %.2ff, %.2fF, %.2ef\n"
      )
      "\n"
      , .123f, -0.123F, 1234e-25f
    );

    /* Literály typu float obvykle není potřeba používat, protože běžné racionální
       literály jsou typu double. Je ale třeba počítat s tím, že přiřazení takové
       konstanty do proměnné typu float dojde ke zmenšení přesnosti.

       Literály s příponou se mohou hodit například pro porovnávání v podmínkách.
    */

    printh(
      PODSEKCE("Typ double")
      "- Výchozí typ, bez přípon\n"
      "\n"
      KOD(
      "  %.2lf, %.2lf, %.2le\n"
      )
      "\n"
      , .123, -0.123, 1234e-251
    );

    printh(
      PODSEKCE("Typ long double")
      "- Desetinné číslo s příponou l, L\n"
      "\n"
      KOD(
      "  %.2Lfl, %.2LfL, %.2LeL\n"
      )
      "\n"
      "- Pozor, neplést s celočíselnou konstantou typu long!\n"
      "  Hodnota " KK("123.0L") " není stejného typu jako " KK("123L") ".\n"
      "\n"
      , .123l, -0.123L, 1234e-4900L
    );
  }

  konecKapitoly(__func__);
}



////////////////////////////////////////////////////////////////////////////////
void racionalniTypyProblemy(void)
{
  zacatekKapitoly("Racionální typy - na co si dát pozor");

  printh(
    "- Vlivem zaokrouhlavání matematické operace nemusí fungovat správně.\n"
    "- Důvodem není jazyk C, ale způsob uložení racionálních čísel v počítači.\n"
    "- Tohle platí pro všechny programovací jazyky.\n"
    "\n"
    );

  if (navratNeboSekce("Nepřesné vyjádření hodnot")) return;
  if (pokracovat())
  {
    /* Reálná čísla nejde na počítači ukládat přesně, pouze s určitou přesností.
    Na počítači nejde uložit nekonečný rozvoj reálného čísla (např.
    u iracionálních čísel), proto se ukládá pouze několik prvních desetinných
    míst. Formálně tedy nejde o reálná čísla, ale o racionální (a i z těch umíme
    v počítači zobrazit jen omezené množství).

    Racionální číslo se ukládá v normalizovaném tvaru ve formátu
    mantisa * 2^exponent. Normalizovaný tvar znamená, že se celá část čísla
    posune za desetinnou čárku a příslušným způsobem se upraví exponent.

    Mantisa je u typu double 52 bitů, kde se ukládá desetinná část čísla, která
    se následně násobí dvojkou umocněnou uloženým exponentem (11 bitů). Z toho
    vyplývá, že racionální datové typy umí uložit pouze omezený počet desetinných
    míst (u typu double to je asi 15, u typu float jenom 6 desítkových cifer).

    Některá desetinná čísla nelze ve dvojkové soustavě vyjádřit přesně. Příkladem
    je desítková hodnota 0,1. Ta je ve dvojkové soustavě periodické číslo
    0,000110011... Uložením omezeného počtu bitů z tohoto nekonečného rozvoje
    nutně dojde ke změně hodnoty. Při zpětném převodu na desítkové číslo dostaneme
    o něco menší hodnotu, například 0,09999843465... Jak moc se liší od původní
    přesné hodnoty je dáno délkou použité mantisy.

    */
    printh(
      "- Některé hodnoty nejde vyjádřit přesně => pozor na porovnávání různých typů.\n"
      "- Výraz " KK("0.1f") " je typu " KK("float") ", ale " KK("0.1") " je typu " KK("double") "!\n"
      "- Hodnota 0,1 je ve dvojkové soustavě periodické číslo!\n"
      "\n"
      KOD(
      "  if (0.1f == 0.1)\n"
      "    printf(\"Je to stejné.\\n\");\n"
      "  else\n"
      "    printf(\"Není to stejné.\\n\");\n"
      )
      "\n"
      "  => "
    );
    if (0.1f == 0.1)
      printh("Je to stejné.\n");
    else
      printh("Není to stejné!\n");

    printh(
    "\n"
    "- Rozdíl je vidět ve dvojkové podobě (znaménko|exponent|mantisa):\n"
    "  0.1f => "
    );
    tiskniBityFloat(0.1f);
    printh(
    "\n"
    "  0.1  => "
    );
    tiskniBityDouble(0.1);
    printh(
    "\n"
    );

    printh(
    "\n"
    "- Stejný případ, ale méně zjevný:\n"
      "\n"
      KOD(
    "  float x = 0.1; // tady se výraz 0.1 automaticky přetypuje na float\n"
    "  if (x == 0.1)  // tady se porovnává float a double\n"
    "    printf(\"Je to stejné.\\n\");\n"
    "  else\n"
    "    printf(\"Není to stejné.\\n\");\n"
      )
    "\n"
    "  => "
    );
    float x = 0.1; // tady se výraz 0.1 automaticky přetypuje na float
    if (x == 0.1)  // tady se porovnává float a double
      printh("Je to stejné.\n");
    else
      printh("Není to stejné!\n");

    printh("\n");
  }

  if (navratNeboSekce("Pozor na omezenou přesnost při sčítání")) return;
  if (pokracovat())
  {
    /* Racionální typy mají omezenou přesnost.

    1e8f + 1.0f == 1e8f
    Mantisa hodnoty typu float pojme zhruba 7 desetinných míst. Přičtení jedničky
    k hodnotě 1e8f nepovede k žádné změně, protože se přičítá k řádu, který už
    v hodnotě 1e8f není uložen.
    */

    printh(
      "- Součet velkého a malého čísla nefunguje, jak má.\n"
      "\n"
      KOD(
      "  float a = 1e8f;\n"
      "  float b = a + 1.0f;\n"
      "  if (a == b)\n"
      "    printf(\"a == b To je divné, že?\\n\");\n"
      "  else\n"
      "    printf(\"a != b Tohle očekávám.\\n\");\n"
      )
      "\n"
      "  => "
    );
    float a = 1e8f;
    float b = a + 1.0f;
    if (a == b)
      printh("a == b To je divné, že?\n");
    else
      printh("a != b Tohle očekávám.\n");

    printh("\n");
  }

  if (navratNeboSekce("Pozor na porovnávání desetinných čísel")) return;
  if (pokracovat())
  {
    /* Nepřesné vyjádření desetinných čísel může způsobit zacyklení.

    => V podmínkách cyklu NIKDY nepoužívat ostré porovnávání ==, !=. Lepší je
       použít výrazy s <, >, <=, >=.

    */
    printh(
      "- Ostré porovnávání desetinných čísel v podmínce cyklu => často se zacyklí.\n"
      "\n"
      KOD(
      "  float c = 0; printf(\"%%.8g\", c);\n"
      "  while (c != 1.0) {\n"
      "    printf(\", %%.8g\", c);\n"
      "    c += 0.1;\n"
      "    if (c > 2.0) break; // tohle by snad nemělo být potřeba...\n"
      "  }\n"
      )
      "\n"
      "  => "
    );
    float c = 0;
    printh("%.8g", c);
    while (c != 1.0)
    {
      c += 0.1;
      printh(", %.8g", c);
      if (c > 2.0) break;
    }

    printh("\n  => Aha! Nebýt break, zacyklilo by se to.\n");
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void typChar(void)
{
  zacatekKapitoly("Typ char - malé celé číslo nebo znak");

  printh(
    SEKCE("Základní údaje")
    "- Počet bitů: %d bitů\n"
    "- Počet bajtů: " KK("sizeof(char)") " = %zu B (prakticky všude)\n"
    "\n"
    SEKCE("Velikosti z limits.h")
    "\n"
    "- Znaménkový typ: " KK("char") ", " KK("signed char") "\n"
    "- Minimální hodnota: " KK("CHAR_MIN = %d") "\n"
    "- Maximální hodnota: " KK("CHAR_MAX = %d") "\n"
    "\n"
    "- Neznaménkový typ: " KK("unsigned char") "\n"
    "- Minimální hodnota: " KK("0") "\n"
    "- Maximální hodnota: " KK("UCHAR_MAX = %u") "\n"
    "\n"
    ,
    CHAR_BIT,
    sizeof(char),
    CHAR_MIN,
    CHAR_MAX,
    UCHAR_MAX
  );

  if (navratNeboSekce("Zápis znakových konstant - znakové literály")) return;
  if (pokracovat())
  {
    /* Typ char se používá ve dvou významech - malé celé číslo v rozsahu
    < -127, +128 >, nebo znak.

    Typ char má velikost 1 bajt (prakticky všude).

    Znak JE číslo - není je nutné mezi sebou převádět. Znaková hodnota je číslo
    odpovídající pozici v ASCII tabulce. Například konstanta 'A' je hodnotou 65.

    Typ char je znaménkový typ! Chcete-li používat neznaménkovou hodnotu v rozsahu
    < 0, 255 >, musíte použít typ unsigned char.

    Nad typem char funguje běžná celočíselná aritmetika (+, -, *, /, %).
    */

    printh(
      "- V jednoduchých uvozovkách - neplést se stringy!!\n"
      "- Znakový literál má hodnotu typu " KK("int") ". Přiřazením do proměnné se přetypuje.\n"
      "- Hodnota znaku je hodnotou kódu v ASCII tabulce.\n"
      "\n"
      KOD(
      "  '%c' == %d\n"
      )
      "\n"
      , 'A', 'A'
    );
  }

  if (navratNeboSekce("Typ char je číselný typ.")) return;
  if (pokracovat())
  {
    char a = 25, b = -10;
    char c = a + b;
    printh(
      KOD(
      "  char a = %d, b = %d;\n"       // tisknu jako číselné hodnoty, ne znaky
      "  char c = a + b;\n"
      )
      "\n"
      "  => c: %d + %d => c: %d\n"
      "\n"
      , a, b
      , a, b, c
    );
  }

  if (navratNeboSekce("Typ char jde interpretovat jako znakový typ.")) return;
  if (pokracovat())
  {
    // Znak je totéž, co hodnota pozice v ASCII tabulce.
    char a = 'A';
    char b = 99;
    printh(
      KOD(
      "  char a = 'A';\n"
      "  char b = 99;\n"
      )
      "\n"
      "  a: '%c', což je hodnota %d\n"
      "  b: %d, což je znak '%c'\n"
      "\n"
      , a, a
      , b, b
    );
  }

  if (navratNeboSekce("Znaky s diakritikou jsou záporné!")) return;
  if (pokracovat())
  {
    /* Znaky s diakritikou jsou záporná čísla. Pozor na to zvláště při indexování
    polí - pak je dobré hodnotu nejprve přetypovat, např. pole[(unsigned)'á'].
    */
    char a = (char) 225; // v CP-1250 i v ISO-8859-2 je to dlouhé a (tedy á)
    printh(
      "- Pozor! Znaky ve druhé půlce ASCII tabulky (různé diakritiky) jsou\n"
      "  záporná čísla!\n"
      "\n"
      KOD(
      "  a = (char) 225; // v CP-1250 i v ISO-8859-2 je to dlouhé a (tedy á)\n"
      )
      "  a: '%c', což je hodnota %d\n"
      "\n"
      "- Poznámka: Tento program je psán v UTF-8, proto zde takový znak není\n"
      "  nic rozpoznatelného.\n"
      "\n"
      , a, a
    );

    /* Tento dokument je psaný v kódování UTF-8, proto tady znak v 8b kódování
    nejde zapsat přímo.

    Tisk české diakritiky v C/C++ na konzoli je vinou firmy Microsoft
    nekonzistentní, proto je lepší se mu vyhnout. MS Windows používá v grafickém
    prostředí kódování češtiny CP-1250, kdežto v textovém terminálu CP-852.
    Zajímavé časy na ně!!! (viz dílo Terryho Pratchetta)

    Pro přepnutí textového terminálu do CP-1250 je potřeba použít funkci
    z modulu locale.h.
    setlocale(LC_ALL, "Czech_Czechia.1250");
    Pro tento projekt to nepoužívám, protože bych musel udržovat zdrojový
    text ve dvou kódováních. Místo toho používám vlastní funkce printg a printh,
    které dělají překódování z UTF-8 do CP-852 za běhu.
    Bližší informace k přepínání lokalizace viz kapitolu Stringy.

    Novější standardy jazyka C umožňují pracovat se širokými znaky a řetězci
    v kódováních Unicode pomocí knihovy wchar.h. Používá se pak typ wchar_t,
    pro řetězce wchar_t * nebo wchar_t pole[]. Široké textové řetězce mají
    předponu L: L"Příliš žluťoučký kůň". Funkce pro práci s nimi mají předponu w,
    např. wprintf.

    Podpora těchto věcí ale ještě není mezi překladači konzistentní.
    */
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void typCharFormatovanyTiskCteniOperace(void)
{
  zacatekKapitoly("Typ char - formátované čtení a zápis + další operace se znaky");

  printh(
    "- Znakový vstup a výstup: " KK("getchar") " a " KK("putchar") "; " KK("fgetc") ", " KK("fputc") "\n"
    "- Formátovaný vstup a výstup: " KK("scanf") ", " KK("printf") "; " KK("fscanf") ", " KK("fprintf") "\n"
    "- Další operace: viz ctype.h \n"
    "\n"
  );

  int c;
  if (navratNeboSekce("Funkce getchar")) return;
  if (pokracovat())
  {
    /* Funkce getchar sice čte ze vstupu po znacích, ale vrací hodnotu typu int.
    Dělá to proto, aby mohla vrátit hodnotu EOF, jejíž hodnota se nevejde do typu
    char.
    */
    printh(
      "- Čtení znaku ze vstupu. Vrací ale typ " KK("int") ", ne " KK("char") "!.\n"
      "- Důvodem je, že vrací hodnotu " KK("EOF") ", když narazí na konce vstupu.\n"
      "  " KK("EOF") " má hodnotu -1 typu " KK("int") ", aby jej šlo odlišit od znaku s ASCII kódem \n"
      "  255, což je ve znaménkovém typu " KK("char") " také hodnota -1.\n"
  // Hodnota -1 typu int má ale znaménkový bit ve 4. bajtu, kdežto -1 typu char
  // má znaménkový bit ve svém 1. (a jediném) bajtu.
      "\n"
      KOD(
      "  int c = getchar();\n"
      )
      "\n"
      "  Zadej libovolný znak a stiskni Enter.\n"
      "  Nebo pro konec vstupu zadej ve Windows Ctrl+Z, v Linuxu Ctrl+D.\n  "
    );

    c = getchar();
    pozerKonecRadku(); // Ošetření, kdyby uživatel zadal i něco dalšího.

    printh(
      "  => c: %d, v ASCII '%c'\n"
      "\n"
      , c, c
    );
    if (c == EOF)
    {
      printh(
        "Zadal jsi " KK("EOF") ". Super! Teď musím hacknout stdin, aby mohl program pokračovat...\n"
        "Pozor! " KK("EOF") " není znak!\n"
        "\n"
      );
      // Tohle nemusíte vědět... Tohle řekne terminálu, že jsem si ten konec rozmyslel...
      clearerr(stdin);
    }
  }

  if (navratNeboSekce("Funkce putchar")) return;
  if (pokracovat())
  {
    /* Funkce putchar je navržena jako protiklad funkce getchar.
    Očekává tedy také parametr typu int.

    Funkce má také návratovou hodnotu. Kdyby při pokusu o zápis došlo k chybě,
    vrátí hodnotu EOF.
    */
    printh(
      "- Tisk znaku na výstup.\n"
      "- Je kompatibilní s " KK("getchar") ", proto očekává argument typu " KK("int") ", ne " KK("char") ".\n"
      "\n"
      KOD(
      "  putchar(c);\n"
      )
      "\n"
      "  =>"
    );
    putchar(c);

    printh(
      "\n"
      "\n"
    );
  }

  if (navratNeboSekce("Čtení znaku pomocí scanf")) return;
  if (pokracovat())
  {
    /* Pozor! Funkce scanf při čtení znaku přečte cokoli včetně bílých znaků!
    To je jediná výjimka mezi formátovými značkami, kdy scanf nepřeskakuje bílé
    znaky.
    */
    printh(
      "- " KK("scanf") " + znacka %%c - čte znak, včetně bílých (mezera, tabulátor, ...)\n"
      "\n"
      KOD(
      "  char znak;\n"
      "  int err = scanf(\"%%c\", &znak);\n"
      )
      "\n"
      "  Zadej mezeru, znak a stiskni Enter.\n"
      "  "
    );
    char znak;
    int err = scanf("%c", &znak);
    pozerKonecRadku();
    printh(
      "  => err: %d, c: %d, v ASCII '%c'\n"
      "\n"
      , err, znak, znak
    );
    if (err == EOF)
      clearerr(stdin);
  }

  if (navratNeboSekce("Přeskočení bílých znaků pomocí scanf")) return;
  if (pokracovat())
  {
    /* Mezera před značkou %c zajistí, že scanf nejprve přeskočí bílé znaky.
       Na tohle je třeba pamatovat, když se používá více značek za sebou.
    */
    printh(
      "- " KK("scanf") " + mezera + %%c - přeskočí bílé znaky\n"
      "\n"
      KOD(
      "  char znak;\n"
      "  int err = scanf(\" %%c\", &znak);\n"
      )
      "\n"
      "  Zadej mezery, znak a stiskni Enter.\n"
      "  "
    );

    char znak;
    int err = scanf(" %c", &znak);
    pozerKonecRadku();

    printh(
      "  => err: %d, c: %d, v ASCII '%c'\n"
      "\n"
      , err, znak, znak
    );
    if (err == EOF)
      clearerr(stdin);
  }

  if (navratNeboSekce("Čtení pole znaků pomocí scanf")) return;
  if (pokracovat())
  {
    /* Zadá-li se ke značce rozměr (%3c) přečte se zadaný počet znaků včetně
    bílých, ale odpovídající argument funkce scanf musí být pole o stejné nebo
    větší délce.

    Pozor! Narozdíl od značky %Ns se v tomto případě za načtenou posloupnost
    znaků nedoplňuje ukončovací nula (znak '\0').
    */
    printh(
      "- " KK("scanf") " + %%Nc - přečte N znaků do pole\n"
      "\n"
      KOD(
      "  char poleznaku[3];\n"
      "  int err = scanf(\"%%3c\", poleznaku);\n"
      )
      "\n"
      "- Pozor! Na rozdíl od značky %%Ns nedoplňuje na konec znak '\\0'.\n"
      "  Zadej alespoň 3 znaky a stiskni Enter.\n"
      "  "
    );

    char poleznaku[3];
    int err = scanf("%3c", poleznaku); // poleznaku bez & - pole je ukazatel
    pozerKonecRadku();

    printh(
      "  => err: %d, poleznaku: %c%c%c\n"
      "\n"
      , err, poleznaku[0], poleznaku[1], poleznaku[2]
    );
    if (err == EOF)
      clearerr(stdin);

    printh(
      "\n"
    );
  }


  if (navratNeboSekce("Operace z ctype.h")) return;
  if (pokracovat())
  {
  /* Hlavičkový soubor ctype.h poskytuje funkce a makra pro převody znaků
     a jejich testování.
  */
    int znak = 'z';
    printh("  znak = '%c';\n", znak);

    znak = toupper(znak);
    printh(
      KOD(
      "  znak = toupper(znak);"
      )
      "\t=> znak: '%c'\n"
    , znak
    );

    znak = tolower(znak);
    printh(
      KOD(
      "  znak = tolower(znak);"
      )
      "\t=> znak: '%c'\n"
      , znak
    );

    printh(
      KOD(
      "  if (islower(znak))\n"
      "    printf(\"znak '%%c' je malé písmeno\\n\", znak);\n"
      )
    );
    if (islower(znak))
      printh("  =>znak '%c' je malé písmeno\n", znak);

    printh(
      "\n"
      "- Podobně jde použít další operace:\n"
      "  " KK("isupper(znak)") " -- testuje velké písmeno\n"
      "  " KK("isalpha(znak)") " -- testuje písmeno a-zA-Z\n"
      "  " KK("isdigit(znak)") " -- testuje číslici 0-9\n"
      "  " KK("isspace(znak)") " -- testuje bílý znak (mezera, \\t, \\r, \\v, \\f, \\n)\n"
      "  " KK("isblank(znak)") " -- testuje řádkový bílý znak (mezera, \\t)\n"
      "  " KK("isprint(znak)") " -- testuje tisknutelný znak\n"
      "\n"
      "- Další viz manuálové stránky.\n"
      "\n"
    );
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void logickyTypInt(void)
{
  zacatekKapitoly("Typ int jako logický typ");

  { printh(
      "- V jazyce C má každý logický výraz hodnotu typu " KK("int") ".\n"
      "- Hodnota 0 je nepravda. Nenulová hodnota je pravda.\n"
      "\n"
      KOD(
      "  if (0) printf(\"0 je pravda\\n\");\n"
      "  else printf(\"0 je nepravda\\n\");\n"
      )
      "\n"
      "  => "
    );

    if (0) printf("0 je pravda\n");
    else printf("0 je nepravda\n");

    printh(
      "\n"
      KOD(
      "  if (-10) printf(\"-10 je pravda\\n\");\n"
      "  else printf(\"-10 je nepravda\\n\");\n"
      )
      "\n"
      "  => "
    );

    if (-10)
      printf("-10 je pravda\n");
    else
      printf("-10 je nepravda\n");

    printh("\n");
  }

  if (navratNeboSekce("Logické operátory")) return;
  if (pokracovat())
  {
    printh(
      "- Jejich výsledkem jsou hodnoty 0 nebo 1 typu " KK("int") ".\n"
    );
    printh("  " KK("10 < 100") "      => %d\n", 10 < 100);
    printh("  " KK("10 <= 100") "     => %d\n", 10 <= 100);
    printh("  " KK("10 > 100") "      => %d\n", 10 > 100);
    printh("  " KK("10 >= 100") "     => %d\n", 10 >= 100);
    printh("  " KK("10 == 100") "     => %d\n", 10 == 100);
    printh("  " KK("10 != 100") "     => %d\n", 10 == 100);
    printh("  " KK("!(10 == 100)") "  => %d\n", !(10 == 100));
    printh("  " KK("1 && 0") "        => %d\n", 1 && 0);
    printh("  " KK("1 || 0") "        => %d\n", 1 || 0);
    printh("  " KK("(0 || 1) && 0") " => %d\n", (0 || 1) && 0); // && má vyšší prioritu než ||, ale doporučuje se používat závorky

    printh("\n");
  }

  if (navratNeboSekce("Logická hodnota jiných typů")) return;
  if (pokracovat())
  {
    printh(
      "- Hodnota jiného typu použitá jako logický výraz se nejprve převede na " KK("int") ".\n"
      "\n"
      KOD(
      "  if (\"Toto je lež.\")\n"
      "    printf(\"Řetězec se vyhodnotí jako pravda.\\n\");\n"
      "  else\n"
      "    printf(\"Řetězec se vyhodnotí jako nepravda\\n\");\n"
      )
      "\n"
      "  => "
      );
      if ("Toto je lež.")
        printh("Řetězec se vyhodnotí jako pravda.\n");
      else
        printh("Řežezec se vyhodnotí jako nepravda\n");

    printh(
      "\n"
      "- Ukazatel je vždy pravda, pouze " KK("NULL") " je nepravda.\n"
      "\n"
      KOD(
      "  if (NULL)\n"
      "    printf(\"NULL se vyhodnotí jako pravda.\\n\");\n"
      "  else\n"
      "    printf(\"NULL se vyhodnotí jako nepravda\\n\");\n"
      )
      "\n"
      "  => "
      );
      if (NULL)
        printh("NULL se vyhodnotí jako pravda.\n");
      else
        printh("NULL se vyhodnotí jako nepravda\n");
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void logickyTypBool(void)
{
  zacatekKapitoly("Logický typ bool");

  /* Typ bool je zaveden od ISO C99 pomocí stdbool.h.

  Je to jen přejmenovaný typ int. Používá se to pro syntaktické zpřehlednění
  kódu, například u návratových hodnot funkcí vracejících logickou hodnotu.

  V hlavičkovém souboru jsou definovány konstanty true = 1 a false = 0.

  POZOR! Nikdy neporovnávejte logický výraz s žádnou z těchto konstant.
  Například hodnota -10 se normálně vyhodnotí jako logická pravda, ale logický
  výraz (-10 == true) není pravdivý!
  */
  printh(
    "- Typ bool je zaveden od ISO C99 pomocí " KK("stdbool.h") ".\n"
    "- Konstanty " KK("true") " a " KK("false") " mají celočíselnou hodnotu.\n"
    "\n"
    KOD(
    "  true = %d\n"
    "  false = %d\n"
    )
    "\n"
    , true, false);

  if (navratNeboSekce("Porovnávání s logickými konstantami")) return;
  if (pokracovat())
  {
    printh(
      "- Porovnávání s " KK("true") " a " KK("false") " je nebezpečné, protože\n"
      "  se ve skutečnosti porovnává s konstantami " KK("1") " a " KK("0") ".\n"
      "\n"
      );
      printh(
        "  " KK("-10") " je logicky pravdivý výraz...\n"
        "  ..., ale " KK("(-10 == true)") " => " KK("(-10 == 1)") " => %d\n"
        , (-10 == true)
      );



    /* Porovnávání s logickou hodnotou je začátečnický zlozvyk. Logický výraz
    má logickou hodnotu, není třeba jej porovnávat s true a false!
    */
      printh(
      "\n"
      "- Porovnání logického výrazu s " KK("true") " a " KK("false") " je zbytečná operace.\n"
      "- Logický výraz už MÁ svou logickou hodnotu.\n"
      "\n"
      KOD(
      "  if ((5 > 3) == true) // Amatérismus! Stačí: if (5 > 3)\n"
      "    printf(\"pravda\\n\");\n"
      "  else\n"
      "    printf(\"nepravda\\n\");\n"
      )
      "\n"
      "  => "
      );
      if ((5 > 3) == true)
        printf("pravda\n");
      else
        printf("nepravda\n");
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void aritmetickeOperatory(void)
{
  zacatekKapitoly("Aritmetické operátory");

  /* Aritmetické operátory jde aplikovat na všechny číselné typy od char po
     long double. Typ výsledku aritmetické operace závisí na použitých
     operandech. Pokud jsou operandy různých číselných typů, bude výsledek vždy
     toho typu, který má větší rozsah. Je to proto, aby nedošlo ke ztrátě
     informace.
     Například
     short + int => int
     char + long => long
     int + float => float
     float + double => double
  */


  char cChar = 50; short cShort = 12000; int cInt = 150100100;
  float cFloat = 123.456e12;
  cShort = cShort + cChar; // výsledek je typu short
  cInt = cInt - cShort;    // výsledek je typu int
  cFloat = cFloat * cChar; // výsledek je typu float

  printh(
    SEKCE("Typ výsledku záleží na operandech")
    "- Aritmetické operátory fungují nad všemi číselnými typy.\n"
    "- Když je výraz složen z různých typů, výsledek je vždy toho největšího typu.\n"
    "\n"
    KOD(
    "  char cChar = 50; short cShort = 12000; int cInt = 150100100;\n"
    "  float cFloat = 123.456e12;\n"
    )
    "\n"
    "  " KK("cShort = cShort + cChar;") " => %hd, výraz (%hd + %hhd) je short\n"
    "  " KK("cInt = cInt - cShort;   ") " => %d, výraz (%d - %hd) je int\n"
    "  " KK("cFloat = cFloat * cChar;") " => %g, výraz (%g * %hhd) je float\n"
    "\n"
    , cShort, 12000, 50
    , cInt, 150100100, cShort
    , cFloat, 123.456e12, cChar
  );

  if (navratNeboSekce("Dělení /")) return;
  if (pokracovat())
  {
    /* Dělení
       Operátor / funguje s celočíselnými typy jako celočíselné dělení. Je-li
       alespoň jeden operand racionální číslo, bude fungovat jako desetinné dělení.

       Pozor!
       double x = 1/3;
       Do x se uloží hodnota 0.0, protože 1/3 je celočíselný výraz s výsledkem 0,
       který se až při přiřazení přetypuje na double.

       Správně je to takto:
       double x = 1.0/3.0;
       Teď bude v x hodnota 0.3333333333333333.

    */

    printh(
    "- celočíselné dělení - jsou-li oba operandy celočíselné\n"
    "- racionální dělení - je-li alespoň jeden operand racionální\n"
    "\n"
    );
    cInt = 4 / 3;         // => 1    - celočíselné dělení
    cFloat = 4.0f / 3.0f; // => 1.33 - racionalni dělení
    cFloat = 4.0f / 3;    // => 1.33 - racionalni dělení
    cFloat = 4 / 3;       // => 1.00 - celočíselné dělení! - výsledek se přetypuje na float

    printh(
    "  " KK("cInt = 4 / 3;        ") " => %d    - celočíselné dělení\n"
    "  " KK("cFloat = 4.0f / 3.0f;") " => %.2f - racionální dělení\n"
    "  " KK("cFloat = 4.0f / 3;   ") " => %.2f - racionální dělení\n"
    "  " KK("cFloat = 4 / 3;      ") " => %.2f - POZOR! celočíselné dělení!\n"
    "\n"
    " - Poslední výraz bude po provedení celočíselného dělení přetypován na float.\n"
    "\n"
    ,
    cInt,
    4.0f / 3.0f,
    4.0f / 3,
    cFloat
    );
  }

  if (navratNeboSekce("Modulo %")) return;
  if (pokracovat())
  {
    /* Modulo % - zbytek po dělení
       Operátor % funguje jen s celočíselnými typy.

       Často se používá pro kruhové indexování pole omezené délky.
       pole[i%N]

       Pozor! Výsledek má stejné znaménko jako první operand. Pokud je tedy první
       operand záporný, bude i výsledek záporný. Je-li potřeba se něčemu takovému
       vyhnout, je potřeba první operand převést na absolutní hodnotu funkcí abs,
       případně jejími variantami labs a llabs.

    */
    printh(
    "- Zbytek po dělení\n"
    "- Funguje jen s celočíselnými operandy.\n"
    "- Znaménko výsledku je určeno znaménkem prvního operandu.\n"
    "\n"
    );
  //  cFloat = 15.0f % 7.0f; // tohle nejde -> syntakticka chyba
    cInt = 15 % 7;   // => 1
    cInt = 15 % -7;  // => 1
    cInt = -15 % 7;  // => -1
    cInt = -15 % -7; // => -1
    printh(
    "  " KK("cInt = 15 %% 7;  ") " => %d\n"
    "  " KK("cInt = 15 %% -7; ") " => %d\n"
    "  " KK("cInt = -15 %% 7; ") " => %d\n"
    "  " KK("cInt = -15 %% -7;") " => %d\n"
    "\n"
    , 15 % 7
    , 15 % -7
    , -15 % 7
    , -15 % -7
    );
  }

  if (navratNeboSekce("Unární inkrementace a dekrementace")) return;
  if (pokracovat())
  {
    printh(
    "- Přičtení a odečtení 1: " KK("++") ", " KK("--") "\n"
    "- Fungují s jakýmkoli číselným typem.\n"
    "- Fungují jen nad L-hodnotou (proměnnou), ne nad konstantou.\n"
    "- Vrací výsledek a zároveň mění hodnotu proměnné!\n"
    "- POZOR! Nepoužívat v jednom výrazu opakovaně nad stejnou proměnnou!\n"
    "\n"
    );

    cInt = 100;
    int vysledek = cInt++;

    printh(
      PODSEKCE("Postfixová verze X++, X--")
      "- Nejprve vrací hodnotu proměnné a až pak ji změní o 1.\n"
      "\n"
      KOD(
      "  cInt = 100;\n"
      "  int vysledek = cInt++;\n"
      )
      "\n"
      "  => vysledek: %d, cInt: %d\n"
      "\n"
      , vysledek, cInt
    );

    cInt = 100;
    vysledek = ++cInt;
    printh(
      PODSEKCE("Prefixová verze ++X, --X")
      "- Nejprve změní hodnotu o 1 a pak vrací hodnotu změněné proměnné.\n"
      "- O něco efektivnější než postfixová verze.\n"
      // Efektivnější proto, že nemusí uchovávat vnitřní proměnnou.
      // Optimalizace moderních překladačů ale tento rozdíl smaže.
      "\n"
      KOD(
      "  cInt = 100;\n"
      "  int vysledek = ++cInt;\n"
      )
      "\n"
      "  => vysledek: %d, cInt: %d\n"
      "\n"
      , vysledek, cInt
    );
  }

  if (navratNeboSekce("Výrazy s nedefinovaným výsledkem")) return;
  if (pokracovat())
  {
// Vypínám varování/chybové hlášení, protože to chci demonstrovat.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wsequence-point"
    cFloat = 10.2; cInt = 10;
    cFloat = cFloat--;       // Překladač hlásí varování.
    cInt = cInt+++cInt++;    // Překladač hlásí varování.
  //  cInt = cInt+++++cInt;  // Tohle je chyba -> cInt++ ++ + cInt
    printh(
      "- POZOR na výrazy s nedefinovaným výsledkem\n"
      "- Proměnná modifikovaná pomocí ++ a -- se nesmí v jednom výrazu\n"
      "  vyskytnout vícekrát.\n"
      "- Není zaručeno pořadí zápisů do proměnné!\n"
      "\n"
      KOD(
      "  cFloat = 10.2; cInt = 10;\n"
      "  cFloat = cFloat--;         // Překladač hlásí varování.\n"
      "  cInt = cInt+++cInt++;      // Překladač hlásí varování.\n"
      "  //  cInt = cInt+++++cInt;  // Tohle je chyba -> cInt++ ++ + cInt\n"
      )
      "\n"
      "  => cFloat = %g  (může být 9.2 i 10.2)\n"
      "  => cInt = %d      (může být 20, 21 i 22)\n"
      "\n"
      , cFloat, cInt
    );
#pragma GCC diagnostic pop
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void bitoveOperatory(void)
{
  zacatekKapitoly("Bitové operátory");

  printh(
    "- Provádějí logické operace s jednotlivými bity celočíselných hodnot paralelně.\n"
    "- Výsledkem je celočíselná, nikoli logická hodnota!\n"
    "\n"
    );

  printh(
    "  Zadej dvě hodnoty šestnáctkově oddělené mezerou (0-FF - 1 bajt).\n"
    "  První hodnotu zadej v rozsahu 0--7F (např. 1D), druhou  80--FF (např. A7).\n"
    "  "
  );

  unsigned char a, b;
  if (scanf("%hhx %hhx", &a, &b) != 2)
  {
    a = 0x1D; b = 0xA7;
    printh("Na vstupu je něco špatně, zadávám a: %hhu, b: %hhu\n", a, b);
  }
  pozerKonecRadku(); // Odstraní ze vstupu řádek, kdyby uživatel zadal něco špatně.

  printh("a = signed (%hhd)_10, unsigned (%hhu)_10, (%hhX)_16\n", (signed)a, a, a);
  printh("b = signed (%hhd)_10, unsigned (%hhu)_10, (%hhX)_16\n", (signed)b, b, b);
  printh("\n");



  if (navratNeboSekce("Bitový OR: a | b")) return;
  if (pokracovat())
  {
    printh(  "        %3hhu = ", a);
    tiskni8Bitu(a);
    printh("\n        %3hhu = ", b);
    tiskni8Bitu(b);
  //    printh("\n            --------\n");
    printh("\n  %3hhu | %3hhu = ", a, b);
    tiskni8Bitu(a | b);
    printh(" = %hhu\n", a | b);
    printh("\n");
  }

  if (navratNeboSekce("Bitový AND: a & b")) return;
  if (pokracovat())
  {
    printh(  "        %3hhu = ", a);
    tiskni8Bitu(a);
    printh("\n        %3hhu = ", b);
    tiskni8Bitu(b);
  //    printh("\n            --------\n");
    printh("\n  %3hhu & %3hhu = ", a, b);
    tiskni8Bitu(a & b);
    printh(" = %hhu\n", a & b);
    printh("\n");
  }

  if (navratNeboSekce("Bitový NOT: ~a")) return;
  if (pokracovat())
  {
    printh(  "   %3hhu = ", a);
    tiskni8Bitu(a);
  //    printh("\n        --------\n");
    printh("\n  ~%3hhu = ", a);
    tiskni8Bitu(~a);
    printh(" = %hhu\n", ~a);
    printh("\n");
  }

  if (navratNeboSekce("Bitový XOR: a ^ b")) return;
  if (pokracovat())
  {
    printh(  "        %3hhu = ", a);
    tiskni8Bitu(a);
    printh("\n        %3hhu = ", b);
    tiskni8Bitu(b);
  //    printh("\n            --------\n");
    printh("\n  %3hhu ^ %3hhu = ", a, b);
    tiskni8Bitu(a ^ b);
    printh(" = %hhu\n", a ^ b);
    printh("\n");
  }

  if (navratNeboSekce("Bitový posuv o n bitů vpravo: b >> n")) return;
  if (pokracovat())
  {
    /* Bitový posuv vpravo posouvá jednotlivé bity čísla doprava, přičemž
       zleva doplňuje nuly u unsigned typů, nebo hodnotu znaménkového bitu
       u znaménkových typů (0 u kladných, 1 u záporných hodnot).
    */
    printh(  "       %3hhu = ", b);
    tiskni8Bitu(a);
    printh("\n  %3hhu >> 3 = ", b);
    tiskni8Bitu((signed char)b >> 3);   printh(" signed char");
    printh("\n  %3hhu >> 3 = ", (unsigned char)b);
    tiskni8Bitu((unsigned char)b >> 3);  printh(" unsigned char\n");
    printh("\n");
  }

  if (navratNeboSekce("Bitový posuv o n bitů vlevo: b << n")) return;
  if (pokracovat())
  {
    /* Bitový posun vlevo posouvá bity doleva, přičemž zprava doplňuje nuly.
       Nuly se doplňují jak u znaménkových tak u neznaménkových hodnot.
    */
    printh(  "       %3hhu = ", b);
    tiskni8Bitu(a);
    printh("\n  %3hhu << 3 = ", b);
    tiskni8Bitu((signed char)b << 3);   printh(" signed char");
    printh("\n  %3hhu << 3 = ", (unsigned char)b);
    tiskni8Bitu((unsigned char)b << 3);  printh(" unsigned char\n");
    printh("\n");
  }

  konecKapitoly(__func__);
}

////////////////////////////////////////////////////////////////////////////////
void pretypovaniVyrazu(void)
{
  zacatekKapitoly("Přetypování výrazu");

  printh(
    "- Pozor! Přetypování může způsobit drsné chyby.\n"
    "- Přetypováním na sebe programátor bere veškerou odpovědnost.\n"
    "- Používejte to, jenom když víte, co děláte!\n"
    "\n"
    "- Přetypování potlačí hlášení překladače o typových chybách!\n"
    "- Pokud to děláte jen proto, abyste se takového hlášení zbavili\n"
    "  a nemáte pro to jiné ospravedlnění, koledujete si o závažné problémy!\n"
    "- Chybová hlášení jsou váš přítel, ne obtížný hmyz...\n"
    "\n"
    "- Operátor přetypování mění pohled překladače na výraz.\n"
    "- U číselných typů může provádět úpravu výsledné (nikoli původní) hodnoty\n"
    "  tak, aby to dávalo smysl, např. " KK("float") " -> " KK("int") " ořízne desetinnou část.\n"
    "- Nelze jej použít na levé straně přiřazovacího příkazu.\n"
    "\n"
  );

  if (navratNeboSekce("Syntaxe")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  (typ)výraz\n"
      )
      "\n"
      "- Výsledný typ výrazu je dán typem v závorkách.\n"
      "- Přetypovací operátor má prioritu jako unární operátory.\n"
      "- Vyhodnocuje se zprava doleva.\n"
      "\n"
      "  " KK("(int)2.5 + 3.6  ") " => " KK("2 + 3.6   ") " => %g (celý výraz je typu double!)\n"
      "  " KK("(int)(2.5 + 3.6)") " => " KK("(int)(6.1)") " => %d (celý výraz je typu int)\n"
      "\n"
      , (int)2.5 + 3.6
      , (int)(2.5 + 3.6)
    );
  }

  if (navratNeboSekce("Přetypování signed - unsigned typ")) return;
  if (pokracovat())
  {
    /* Přetypování signed - unsigned
       U příbuzných typů nemění samotnou hodnotu. Pouze se změní způsob
       interpretace jednotlivých bitů výsledku výrazu.

       Například přetypování hodnoty typu int na usigned int vůbec nezmění bity
       původní hodnoty, pouze se změní způsob interpretace nejvyššího bitu -
       u typu int jde o znaménko, u typu unsigned int je to nejvyšší řád hodnoty.
       Podobně je to u všech celočíselných typů.

       char a = -1;                        // bitově 11111111 -> -1 (je to doplňkový kód)
       unsigned char b = (unsigned char)a; // bitově 11111111 -> 255
    */
    printh(
      "- Nemění bitovou podobu číselné hodnoty.\n"
      "- Mění interpretaci hodnoty a chování aritmetických operátorů.\n"
      "\n"
    );
    char a = -1;
    unsigned char b = (unsigned char) a;
    printh(
      KOD(
      "  char a = -1;\n"
      "  unsigned char b = (unsigned char) a;\n"
      )
      "\n"
    );
  //  printh("  a = %hhd, b = %hhu\n", a, b);
    printh("  => a: %3hhd => ", a); tiskniNBitu(a, sizeof(a)*8, sizeof(char)*8); printh("\n");
    printh("  => b: %3hhu => ", b); tiskniNBitu(b, sizeof(b)*8, sizeof(char)*8); printh("\n");

    /* Přetypování na větší typ
       Nemění hodnotu výrazu. U znaménkových typů zachovává znaménko. Není třeba
       jej dělat explicitně, protože jej překladač dělá automaticky - nepíše žádné
       varování, protože takové přetypování považuje za bezpečné.
    */
    printh("\n");
  }

  if (navratNeboSekce("Přetypování na větší typ")) return;
  if (pokracovat())
  {
    char c = -10;
    int d = (int)c; // není potřeba
    unsigned int e = (unsigned int)c;
    unsigned int f = (unsigned char)c; // nejprve na uchar, pak samo na uint!
    printh(
      "\n"
      KOD(
      "  char c = -10;\n"
      "  int d = (int)c; // není potřeba\n"
      "  unsigned int e = (unsigned int)c;\n"
      "  unsigned int f = (unsigned char)c; // nejprve na uchar, pak samo na uint!\n"
      )
      "\n"
    );
  //  printh("  c = %hhd, d = %d, e = %u, f = %u\n", c, d, e, f);
    printh("  => c: %10hhd => ", c); tiskniNBitu(c, sizeof(c)*8, sizeof(c)*8); printh("\n");
    printh("  => d: %10d => ",d);    tiskniNBitu(d, sizeof(d)*8, sizeof(d)*8); printh("\n");
    printh("  => e: %10u => ", e);   tiskniNBitu(e, sizeof(e)*8, sizeof(e)*8); printh("\n");
    printh("  => f: %10u => ", f);   tiskniNBitu(f, sizeof(f)*8, sizeof(f)*8); printh("\n");

    /* Přetypování na menší typ.  Může dojít k přetečení!
       Překladač z původní vícebajtové hodnoty vezme jen bity odpovídající
       menšímu typu. Žádné zohledňování znaménka se tu neodehrává.

       Pozor! Překladač to dělá také automaticky a netiskne varování.
       Fun fact: Kvůli tomuto chování byl kdysi jazyk C považován za slabě typový
       jazyk. Dnes je považován za silně typový jazyk - v porovnání s dynamickými
       jazyky jako Python nebo JavaScript.
    */
    printh("\n");
  }

  if (navratNeboSekce("Přetypování na menší typ")) return;
  if (pokracovat())
  {
    int d = -500;
    char c = d;
    printh(
      "- Může dojít ke ztrátě informace (přetečení).\n"
      "\n"
      KOD(
      "  int d = -500;\n"
      "  char c = d; // totéž jako (char)d\n"
      )
      "\n"
    );
  //  printh("  d = %d, c = %hhd\n", d, c);
    printh("  => d: %4d => ", d); tiskniNBitu(d, sizeof(d)*8, sizeof(d)*8); printh("\n");
    printh("  => c: %4hhd => ", c); tiskniNBitu(c, sizeof(c)*8, sizeof(c)*8); printh("\n");

    /* Přetypování racionálních typů.
       Při přetypování z většího na menší (double -> float) se zachová hodnota,
       ale může dojít k přetečení.

       Přetypování z menšího na větší je vždy bezpečné.
       Pozor! Nedojde ale ke zvětšení přesnosti původní hodnoty. Například
       hodnota typu float 0.1f získá přetypováním na double v desetinném rozvoji
       další nuly, nikoli další bity nekonečného rozvoje (hodnota 0.1 je ve
       dvojkové soustavě periodické číslo).
    */
    printh("\n");
  }

  if (navratNeboSekce("Přetypování racionálních typů")) return;
  if (pokracovat())
  {
    printh(
      "- Binární formát racionálních typů: znaménko|exponent|mantisa\n"
      "- Rozšiřující přetypování doplní mantisu nulami.\n"
      "- Pozor! Přetypováním na větší typ nedojde ke zvětšení přesnosti původní\n"
      "  hodnoty!\n"
      "\n"
    );
    float m = 0.1f;      // konstanta je typu float
    double n = (float)m; // explicitne, ale to neni potreba
    printh(
      KOD(
      "  float m = 0.1f;      // konstanta je typu float\n"
      "  double n = (float)m; // explicitně, ale to není potřeba\n"
      )
      "\n"
    );
  //  printh("  m = %g, n = %lg\n", m, n);
    printh("  => m: %g => ", m); tiskniBityFloat(m); printh("\n");
    printh("  => n: %lg => ", n); tiskniBityDouble(n); printh("\n\n");

    printh(
      "- Přetypování na menší typ ořízne mantisu => dojde ke ztrátě přesnosti.\n"
      "\n"
    );
    n = 0.1;      // konstanta je typu double
    m = (float)n; // dojde k orezani mantisy
    printh(
      KOD(
      "  double n = 0.1;     // konstanta je typu double\n"
      "  float m = (float)n; // dojde k ořezání mantisy\n"
      )
      "\n"
    );
  //  printh("  n = %lg, m = %g\n", n, m);
    printh("  => n: %lg => ", n); tiskniBityDouble(n); printh("\n");
    printh("  => m: %g => ", m); tiskniBityFloat(m); printh("\n");
    printh("\n");
  }

  if (navratNeboSekce("Přetypování racionálních typů na celočíselné")) return;
  if (pokracovat())
  {
    /* Přetypování racionálních typů na celočíselné.
       Zde dochází k odseknutí desetinné části hodnoty. Pokud se celá část čísla
       nevejde do přiděleného typu, dojde k přetečení! Toto nikdy nepoužívejte
       pro zaokrouhlování. K tomu slouží funkce round(), floor() a trunc()
       z math.h.
    */

    printh(
      "- Dojde k odseknutí desetinné části čísla.\n"
      "- Je-li celá část čísla větší než pojme cílový typ, dojde k přetečení.\n"
      "\n"
    );
    float p = 3.55f;
    int i = p; // (int) p
    double q = 1.23e10;
    short j = q; // (short) q (dojde k přetečení)
    long k = q; // (long) q

    printh(
      "  " KK("float p = 3.55f;         ") " => p=%g\n"
      "  " KK("int i = p; // (int) p    ") " => i=%d\n"
      "\n"
      "  " KK("double q = 1.23e10;      ") " => q=%lg\n"
      "  " KK("short j = q; // (short) q") " => j=%hd (došlo k přetečení)\n"
      "  " KK("long k = q;  // (long) q ") " => k=%ld\n"
      "\n"
      , p, i
      , q, j, k
    );
  }

  if (navratNeboSekce("Lepší je zaokrouhlování")) return;
  if (pokracovat())
  {
    printh(
      "- Obvykle je lepší je použít funkce pro zaokrouhlování.\n"
      "- Vrací výsledek racionálního typu.\n"
      "- Nehrozí nebezpečí přetečení hodnoty.\n"
      "\n"
    );
    printh(
    "  " KK("round(3.55)") " => %.1lf   " KK("round(-3.55)") " => %.1lf\n"
    "  " KK("floor(3.55)") " => %.1lf   " KK("floor(-3.55)") " => %.1lf\n"
    "  " KK("trunc(3.55)") " => %.1lf   " KK("trunc(-3.55)") " => %.1lf\n"
    "\n"
    "  " KK("round(3.2)") " => %.1lf   " KK("round(-3.2)") " => %.1lf\n"
    "  " KK("floor(3.2)") " => %.1lf   " KK("floor(-3.2)") " => %.1lf\n"
    "  " KK("trunc(3.2)") " => %.1lf   " KK("trunc(-3.2)") " => %.1lf\n"
    "\n"
    , round(3.55), round(-3.55)
    , floor(3.55), floor(-3.55)
    , trunc(3.55), trunc(-3.55)
    , round(3.2), round(-3.2)
    , floor(3.2), floor(-3.2)
    , trunc(3.2), trunc(-3.2)
    );
  }

  konecKapitoly(__func__);
}





////////////////////////////////////////////////////////////////////////////////
/** Ukázková funkce, která používá komplexní číslo jako návratový typ. */
double complex cplus(double complex a, double complex b)
{
  return a + b;
}

/** Naformátuje komplexní číslo do algebraického tvaru.*/
char * komplexATvar(char * str, double complex z)
{
  sprintf(str, "%g%+g*I", creal(z), cimag(z));
  return str;
}

void komplexniTypyPrehled(void)
{
  char str1[50]; // pomocné pole pro tisk komplexních čísel
  char str2[50]; // pomocné pole pro tisk komplexních čísel
  char str3[50]; // pomocné pole pro tisk komplexních čísel
  char str4[50]; // pomocné pole pro tisk komplexních čísel
  char str5[50]; // pomocné pole pro tisk komplexních čísel
  char str6[50]; // pomocné pole pro tisk komplexních čísel

  zacatekKapitoly("Komplexní čísla - přehled");

  printh(
    "- Je třeba použít " KK("#include <complex.h>") "\n"
    "- Je třeba použít " KK("#include <math.h>") "\n"
    "- Zavedeno od standardu C99 => překlad s přepínačem " KK("-std=c99") "\n"
    "- Zároveň je nutno použít přepínač " KK("-lm") " (blíže viz zdrojový text).\n"
/* Přepínač -lm slouží k přilinkování matematické knihovny.
   Přepínač -lm se musí použít až za zdrojovým souborem:
   gcc -std=c99 soubor.c -lm

   V Code::Blocks je tento přepínač nutné přidat takto:
   Project -> Build options...-> Jméno projektu -> Linker settings
   -> Other linker options: sem vepsat -lm
*/
    "- Fungují běžné aritmetické operace " KK("+") ", " KK("-") ", " KK("*") ", " KK("/") ".\n"
    "- Další operace jsou realizovány pomocí funkcí z " KK("complex.h") ".\n"
    "\n"
  );

  if (navratNeboSekce("Deklarace komplexní proměnné")) return;
  if (pokracovat())
  {
    printh(
      "\n"
      KOD(
      "  <číselný typ> complex identifikátor\n"
      )
      "\n"
      "  - Za <číselný typ> jde dosadit typy " KK("double") " (výchozí), " KK("float") " a " KK("long double") ",\n"
      "    ale také celočíselné typy " KK("int") ", " KK("short") ", " KK("long") ", ..., pokud to dává smysl.\n"
      "\n"
    );
  }

  if (navratNeboSekce("Inicializace hodnoty")) return;
  if (pokracovat())
  {
    /* I - imaginární jednotka */
    double complex z = 2.5 -5.6 * I;

    printh(
      "\n"
      KOD(
      "  double complex z = %s;\n"
      )
      "\n"
      "- Konstanta I představuje imaginární jednotku.\n"
      "\n"
      , komplexATvar(str1, z)
    );
  }

  if (navratNeboSekce("Algebraický tvar")) return;
  if (pokracovat())
  {
    /*
       %+g - vytiskne double vždy se znaménkem + nebo -
    */
    double complex z = 2.5 -5.6 * I;
    double x = creal(z); // souřadnice x - reálná
    double y = cimag(z); // souřadnice y - imaginární

    printh(
      "- " KK("creal(z)") ": vrací reálnou část čísla (jako " KK("double") ")\n"
      "- " KK("cimag(z)") ": vrací imaginární část čísla (jako " KK("double") ")\n"
      "\n"
      KOD(
      "  double x = creal(z); // souřadnice x - reálná\n"
      "  double y = cimag(z); // souřadnice y - imaginární\n"
      )
      "\n"
      "  => z: %s\n"
      "  => x: %g (reálná část)\n"
      "  => y: %g (imaginární část)\n"
      "\n"
      , komplexATvar(str1, z)
      , x
      , y
    );
  }

  if (navratNeboSekce("Goniometrický tvar")) return;
  if (pokracovat())
  {
    double complex z = 2.5 -5.6 * I;
    double absz = cabs(z); // absolutní hodnota
    double fiz = carg(z);  // úhel fí

    printh(
      "- " KK("cabs(z)") ": Vrací absolutní hodnotu komplexního čísla - vzdálenost od počátku.\n"
      "- " KK("carg(z)") ": Vrací úhel fí v mezích < -M_PI, +M_PI >.\n"
      "\n"
      KOD(
      "  double absz = cabs(z); // absolutní hodnota\n"
      "  double fiz = carg(z);  // úhel fí\n"
      )
      "\n"
      "  => z: %g * e^(%g * I)\n"
      "  => absz: %g\n"
      "  => fiz: %g\n"
      "\n"
      , absz
      , fiz
      , absz
      , fiz
    );
  }

  if (navratNeboSekce("Aritmetické operátory")) return;
  if (pokracovat())
  {
    double complex a = 2.2 +1.2 * I;
    double complex b = 1.1 -0.5 * I;
    double complex aplusb = a + b;
    double complex aminusb = a - b;
    double complex akratb = a * b;
    double complex adelenob = a / b;

    printh(
      "- Vracejí opět komplexní výsledek.\n"
      "\n"
      KOD(
      "  double complex a = %s;\n"
      "  double complex b = %s;\n"
      )
      "  " KK("a + b") " => %s\n"
      "  " KK("a - b") " => %s\n"
      "  " KK("a * b") " => %s\n"
      "  " KK("a / b") " => %s\n"
      "\n"
      , komplexATvar(str1, a)
      , komplexATvar(str2, b)
      , komplexATvar(str3, aplusb)
      , komplexATvar(str4, aminusb)
      , komplexATvar(str5, akratb)
      , komplexATvar(str6, adelenob)
    );
  }

  if (navratNeboSekce("Návratový typ funkce")) return;
  if (pokracovat())
  {
    double complex a = 2.2 +1.2 * I;
    double complex b = 1.1 -0.5 * I;
    double complex zz = cplus(a, b);

    printh(
      "- Typ komplexní číslo může být návratovým typem funkce.\n"
      "\n"
      KOD(
      "  double complex cplus(double complex a, double complex b)\n"
      "  {\n"
      "    return a + b;\n"
      "  }\n"
      "  ...\n"
      "  double complex a = %s;\n"
      "  double complex b = %s;\n"
      "  double complex zz = cplus(a, b);\n"
      )
      "\n"
      "  => zz = %s\n"
      "\n"
      , komplexATvar(str1, a)
      , komplexATvar(str2, b)
      , komplexATvar(str3, zz)
    );
  }

  konecKapitoly(__func__);
}


////////////////////////////////////////////////////////////////////////////////
void komplexniTypyOperace(void)
{
  char str1[50]; // pomocné pole pro tisk komplexních čísel
  char str2[50]; // pomocné pole pro tisk komplexních čísel
  char str3[50]; // pomocné pole pro tisk komplexních čísel
  char str4[50]; // pomocné pole pro tisk komplexních čísel
  char str5[50]; // pomocné pole pro tisk komplexních čísel
  char str6[50]; // pomocné pole pro tisk komplexních čísel
  char str7[50]; // pomocné pole pro tisk komplexních čísel

  zacatekKapitoly("Komplexní čísla - operace");

  printh(
    KOD(
    "#include <math.h>\n"
    "#include <complex.h>\n"
    )
    "- V hlavičkovém souboru jsou definovány operace s komplexními čísly.\n"
    "- Zároveň je nutno použít přepínač -lm (blíže viz zdrojový text).\n"
/* Přepínač -lm slouží k přilinkování matematické knihovny.
   Přepínač -lm se musí použít až za zdrojovým souborem:
   gcc -std=c99 soubor.c -lm

   V Code::Blocks je tento přepínač nutné přidat takto:
   Project -> Build options...-> Jméno projektu -> Linker settings
   -> Other linker options: sem vepsat -lm
*/
    "\n"
    "Tři verze funkcí podle typu\n"
    "- " KK("double complex") "\n"
    "  => bez přípony, např. " KK("creal, cimag") ".\n"
    "  => vrací " KK("double") " nebo " KK("double complex") "\n"
    "\n"
    "- " KK("float complex") "\n"
    "  => přípona f, např. " KK("crealf, cimagf") "\n"
    "  => vrací " KK("float") " nebo " KK("float complex") "\n"
    "\n"
    "- " KK("long double complex") "\n"
    "  => přípona l, např. " KK("creall, cimagl") "\n"
    "  => vrací " KK("long double") " nebo " KK("long double complex") "\n"
    "\n"
  );

  if (navratNeboSekce("Proměnné typu komplexní číslo")) return;
  if (pokracovat())
  {
    double complex x = 0.1 +0.23 * I;
    double complex z = 2.4 -0.61 * I;

    printh(
      "\n"
      KOD(
      "  double complex x = %s;\n"
      "  double complex z = %s;\n"
      )
      "\n"
      , komplexATvar(str1, x)
      , komplexATvar(str2, z)
    );
  }

  if (navratNeboSekce("Souřadnice")) return;
  if (pokracovat())
  {
    double complex x = 0.1 +0.23 * I;
    printh(
      PODSEKCE("Algebraický tvar")
      "  => " KK("creal(x)") ": %g\t- reálná složka\n"
      "  => " KK("cimag(x)") ": %g\t- imaginární složka\n"
      "\n"
      , creal(x)
      , cimag(x)
    );

    printh(
      PODSEKCE("Goniometrický tvar")
      "  => x: %s = %g * e^(%g * I)\n"
      "  => " KK("cabs(x)") ": %g\t- absolutní hodnota |x|\n"
      "  => " KK("carg(x)") ": %g\t- úhlel fí (v radiánech)\n"
      "\n"
      , komplexATvar(str1, x)
      , cabs(x)
      , carg(x)
      , cabs(x)
      , carg(x)
    );
  }

  if (navratNeboSekce("Matematické funkce")) return;
  if (pokracovat())
  {
    double complex x = 0.1 +0.23 * I;
    double complex z = 2.4 -0.61 * I;
    double complex logx = clog(x);
  //  double complex log2x =  clog2(x);   // zatím v C99 není
  //  double complex log10x = clog10(x);  // zatím v C99 není
    double complex expx = cexp(x);
    double complex powxz = cpow(x, z);
    double complex sqrtx = csqrt(x);
    double complex conjx = conj(x);

    printh(
      "- Vrací nové komplexní číslo.\n"
      "  => x: %s\n"
      "  => z: %s\n"
      "  => " KK("clog(x)") ": %s\t\t- přirozený logaritmus (o základu e)\n"
  //    "  => clog2(x): %s\t- logaritmus o základu 2\n"   // zatím v C99 není
  //    "  => clog10(x): %s\t- logaritmus o základu 10\n"  // zatím v C99 není
      "  => " KK("cexp(x)") ": %s\t- e na x\n"
      "  => " KK("cpow(x, z)") ": %s\t- mocnina x na z\n"
      "  => " KK("csqrt(x)") ": %s\t- odmocnina z x\n"
      "  => " KK("conj(x)") ": %s\t\t- komplexně sdružené číslo x\n"
      "\n"
      , komplexATvar(str1, x)
      , komplexATvar(str2, z)
      , komplexATvar(str3, logx)
  //    , komplexATvar(str8, log2x)
  //    , komplexATvar(str9, log10x)
      , komplexATvar(str4, expx)
      , komplexATvar(str5, powxz)
      , komplexATvar(str6, sqrtx)
      , komplexATvar(str7, conjx)
    );
  }

  if (navratNeboSekce("Goniometrické funkce")) return;
  if (pokracovat())
  {
    double complex x = 0.1 +0.23 * I;
    double complex sinx = csin(x);
    double complex cosx = ccos(x);
    double complex tanx = ctan(x);

    printh(
      "- Vrací nové komplexní číslo.\n"
      "  => x: %s\n"
      "  => " KK("csin(x)") ": %s\t- sinus\n"
      "  => " KK("ccos(x)") ": %s\t- kosinus\n"
      "  => " KK("ctan(x)") ": %s\t- tangens\n"
      "\n"
      , komplexATvar(str1, x)
      , komplexATvar(str2, sinx)
      , komplexATvar(str3, cosx)
      , komplexATvar(str4, tanx)
    );

    double complex asinx = casin(sinx);
    double complex acosx = cacos(cosx);
    double complex atanx = catan(tanx);

    printh(
      SEKCE("Inverzní goniometrické funkce (cyklometrické)")
      "- Vrací nové komplexní číslo.\n"
      "  => x: %s\n"
      "  => " KK("casin(csin(x))") ": %s\t- arkus sinus\n"
      "  => " KK("cacos(ccos(x))") ": %s\t- arkus kosinus\n"
      "  => " KK("catan(ctan(x))") ": %s\t- arkus tangens\n"
      "\n"
      , komplexATvar(str1, x)
      , komplexATvar(str2, asinx)
      , komplexATvar(str3, acosx)
      , komplexATvar(str4, atanx)
    );
  }

  if (navratNeboSekce("Hyperbolické funkce")) return;
  if (pokracovat())
  {
    double complex x = 0.1 +0.23 * I;
    double complex sinhx = csinh(x);
    double complex coshx = ccosh(x);
    double complex tanhx = ctanh(x);

    printh(
      "- Vrací nové komplexní číslo.\n"
      "  => x: %s\n"
      "  => " KK("csinh(x)") ": %s\t- hyperbolický sinus\n"
      "  => " KK("ccosh(x)") ": %s\t- hyperbolický kosinus\n"
      "  => " KK("ctanh(x)") ": %s\t- hyperbolický tangens\n"
      "\n"
      , komplexATvar(str1, x)
      , komplexATvar(str2, sinhx)
      , komplexATvar(str3, coshx)
      , komplexATvar(str4, tanhx)
    );

    double complex asinhx = casinh(sinhx);
    double complex acoshx = cacosh(coshx);
    double complex atanhx = catanh(tanhx);
    printh(
      SEKCE("Inverzní hyperbolické funkce (hyperbolometrické)")
      "- Vrací nové komplexní číslo.\n"
      "  => x: %s\n"
      "  => " KK("casinh(csinh(x))") ": %s\t- hyperbolometrický sinus\n"
      "  => " KK("cacosh(ccosh(x))") ": %s\t- hyperbolometrický kosinus\n"
      "  => " KK("catanh(ctanh(x))") ": %s\t- hyperbolometrický tangens\n"
      "\n"
      , komplexATvar(str1, x)
      , komplexATvar(str2, asinhx)
      , komplexATvar(str3, acoshx)
      , komplexATvar(str4, atanhx)
    );
  }

  konecKapitoly(__func__);
}
